/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogfind.h"
#include "utilities.h"
#include "sqlite3_plus.h"
#include "bible.h"
#include "worddatabase.h"


FindDialog::FindDialog (Configuration* configuration)
{
  myconfiguration = configuration;

  finddialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (finddialog), "Find");
  gtk_window_set_position (GTK_WINDOW (finddialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (finddialog), TRUE);
  // Next one added to Glade's code.
  gtk_window_set_destroy_with_parent (GTK_WINDOW (finddialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (finddialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox2, TRUE, TRUE, 0);

  label5 = gtk_label_new ("Search for");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 4);

  entry1 = gtk_entry_new ();
  // Next one has been added to Glade's code.
  gtk_entry_set_text (GTK_ENTRY (entry1), myconfiguration->find_searchword.c_str ());
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (entry1), TRUE);

  checkbutton_case = gtk_check_button_new_with_mnemonic ("_Case sensitive");
  gtk_widget_show (checkbutton_case);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), checkbutton_case, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_case), myconfiguration->case_sensitive_search);

  dialog_action_area1 = GTK_DIALOG (finddialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  buttonfind = gtk_button_new_from_stock ("gtk-find");
  gtk_widget_show (buttonfind);
  gtk_dialog_add_action_widget (GTK_DIALOG (finddialog), buttonfind, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (buttonfind, GTK_CAN_DEFAULT);

  buttoncancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (buttoncancel);
  gtk_dialog_add_action_widget (GTK_DIALOG (finddialog), buttoncancel, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (buttoncancel, GTK_CAN_DEFAULT);

  // The next one modified from Glade's code.
  g_signal_connect ((gpointer) buttonfind, "clicked", G_CALLBACK (finddialog_on_buttonfind_clicked), gpointer (this));

  gtk_widget_grab_focus (entry1);
  gtk_widget_grab_default (buttonfind);
}


FindDialog::~FindDialog ()
{
  gtk_widget_destroy (finddialog);
}


int FindDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (finddialog));
}


void FindDialog::finddialog_on_buttonfind_clicked (GtkButton * button, gpointer user_data)
{
  ((FindDialog *) user_data)->on_buttonfind_clicked ();
}


void FindDialog::on_buttonfind_clicked ()
{
  try
  {
    string searchword = gtk_entry_get_text (GTK_ENTRY (entry1));
    myconfiguration->find_searchword = searchword;
    myconfiguration->case_sensitive_search = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_case));
    // Set a flag if the text is still being indexed at present.
    is_indexing = program_is_running ("bibledit.wdb");
    // Connect to database.
    Worddatabase worddatabase (myconfiguration->project);
    sqlite3::connection con (worddatabase.dbfilename ().c_str ());
    // Get the string to search for. Note any apostrophies need to be doubled for SQLite.
    // We need to normalize the search expression, as prescribed, when comparing strings.
    ustring localsearchword;
    localsearchword = searchword;
    localsearchword = localsearchword.normalize ();
    if (!myconfiguration->case_sensitive_search)
      localsearchword = localsearchword.casefold ();
    localsearchword = double_apostrophy (localsearchword);
    // Execute the SQL statement, and read the results.
    sqlite3::reader reader;
    if (myconfiguration->case_sensitive_search) {
      reader = con.executereader ("select book, chapter, verse from 'lines' where line glob ('*%s*');", localsearchword.c_str ());
    } else {
      reader = con.executereader ("select book, chapter, verse from 'lines' where casefolded glob ('*%s*');", localsearchword.c_str ());
    }
    while (reader.read ())
      results.push_back (book_chapter_verse_to_reference (reader.getint32 (0), reader.getint32 (1), reader.getint32 (2)));
    // Close all
    reader.close ();
    con.close ();
  }
  catch (exception & ex)
  {
    cerr << "Error: " << ex.what () << endl;
  }
}
