/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialoggotoreference.h"
#include "utilities.h"
#include "bible.h"


GotoReferenceDialog::GotoReferenceDialog (const string & currentbook,
                                          const string & currentchapter,
                                          const string & currentverse)
{
  // Init variables
  newreference = false;
  oldbook = currentbook;
  oldchapter = currentchapter;
  oldverse = currentverse;

  gotoreferencedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (gotoreferencedialog), "Goto reference");
  gtk_window_set_position (GTK_WINDOW (gotoreferencedialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (gotoreferencedialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (gotoreferencedialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (gotoreferencedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Go to");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 2);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 2);
  gtk_entry_set_activates_default (GTK_ENTRY (entry1), TRUE);

  dialog_action_area1 = GTK_DIALOG (gotoreferencedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (gotoreferencedialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-jump-to");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (gotoreferencedialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) okbutton1, "clicked", G_CALLBACK (on_okbutton1_clicked), gpointer(this));

  gtk_widget_grab_focus (entry1);
  gtk_widget_grab_default (okbutton1);
}


GotoReferenceDialog::~GotoReferenceDialog ()
{
  gtk_widget_destroy (gotoreferencedialog);
}


int GotoReferenceDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (gotoreferencedialog));
}


void
GotoReferenceDialog::on_jump ()
{
  /*
     This interprets the text the user has typed in the entry as a valid reference.
     If needed it will use information of the current reference to complete the info.
     For example, when the user types "1", it is interpreted as verse one of the current
     chapter of the current book. If he types "21 10" it is interpreted as the current
     book, chapter 21 verse 10. And so forth.
   */

  if (reference_discover (oldbook, oldchapter, oldverse, gtk_entry_get_text (GTK_ENTRY (entry1)), newbook, newchapter, newverse)) {
    newreference = true;
  } else {
    newreference = false;
    show_bad_reference ();
  }
}


void
GotoReferenceDialog::show_bad_reference ()
{
  // We did not find the reference: inform the user.

  string message = "No such reference: ";
  message.append (gtk_entry_get_text (GTK_ENTRY (entry1)));
  GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (gotoreferencedialog),
                                              GTK_DIALOG_DESTROY_WITH_PARENT,
                                              GTK_MESSAGE_ERROR,
                                              GTK_BUTTONS_CLOSE,
                                              message.c_str());
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


void GotoReferenceDialog::on_okbutton1_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  ((GotoReferenceDialog *) user_data)->on_jump();
}
