/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogimportnotes.h"
#include "bible.h"
#include "utilities.h"
#include "dialogwait.h"
#include "utilities.h"
#include "dialogunicode.h"
#include "notes_utils.h"
#include "date_time_utils.h"
#include "sqlite3_plus.h"


#define TEMP_FILE "bibledit.import.notes"


ImportNotesDialog::ImportNotesDialog (int dummy)
{
  importnotesdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (importnotesdialog), "Import Notes");
  gtk_window_set_position (GTK_WINDOW (importnotesdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (importnotesdialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (importnotesdialog), 480, 380);
  gtk_window_set_type_hint (GTK_WINDOW (importnotesdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (importnotesdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook), vbox4);

  label13 = gtk_label_new ("You are going to import notes into your notes editor.\n\nThe notes should be in an xml file, such as the file produced\nby Translation Notes Editor. \n\nClick Forward to continue.");
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (vbox4), label13, FALSE, FALSE, 5);
  gtk_misc_set_alignment (GTK_MISC (label13), 0.29, 0.5);

  label1 = gtk_label_new ("label1");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), label1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (notebook), vbox2);

  label6 = gtk_label_new ("Select the file to import");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 10);

  select_directory_button = gtk_button_new ();
  gtk_widget_show (select_directory_button);
  gtk_box_pack_start (GTK_BOX (vbox2), select_directory_button, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (select_directory_button), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label8 = gtk_label_new_with_mnemonic ("/home/joe");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox2), label8, FALSE, FALSE, 0);

  xml_file_label = gtk_label_new ("");
  gtk_widget_show (xml_file_label);
  gtk_box_pack_start (GTK_BOX (vbox2), xml_file_label, FALSE, FALSE, 10);

  label4 = gtk_label_new ("label4");
  gtk_widget_show (label4);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), label4);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (notebook), vbox5);

  label18 = gtk_label_new ("We now know which file to import.\nThe next thing is to check if the file is in Unicode format.\n\nIf it is not in Unicode format, you will be asked to convert it.\n\nNote: The conversion may take time, depending on the size of the file.");
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (vbox5), label18, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  label15 = gtk_label_new ("label15");
  gtk_widget_show (label15);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), label15);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox6);
  gtk_container_add (GTK_CONTAINER (notebook), vbox6);

  label19 = gtk_label_new ("It's now in Unicode format.");
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (vbox6), label19, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  label14 = gtk_label_new ("label14");
  gtk_widget_show (label14);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3), label14);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (notebook), vbox9);
  gtk_container_set_border_width (GTK_CONTAINER (vbox9), 16);

  label24 = gtk_label_new ("Select the element that starts and finishes one complete note.");
  gtk_widget_show (label24);
  gtk_box_pack_start (GTK_BOX (vbox9), label24, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label24), 0.1, 0.5);

  combo_startelement = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_startelement)->popwin),
                     "GladeParentKey", combo_startelement);
  gtk_widget_show (combo_startelement);
  gtk_box_pack_start (GTK_BOX (vbox9), combo_startelement, FALSE, FALSE, 7);

  combo_entry_startelement = GTK_COMBO (combo_startelement)->entry;
  gtk_widget_show (combo_entry_startelement);

  label2 = gtk_label_new ("label2");
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 4), label2);

  table1 = gtk_table_new (9, 2, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (notebook), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 8);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 6);

  label25 = gtk_label_new ("Indicate here which element of the file indicates a certain part of the note.");
  gtk_widget_show (label25);
  gtk_table_attach (GTK_TABLE (table1), label25, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label25), 0, 0.5);

  label26 = gtk_label_new ("References");
  gtk_widget_show (label26);
  gtk_table_attach (GTK_TABLE (table1), label26, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label26), 0, 0.5);

  label27 = gtk_label_new ("Note text");
  gtk_widget_show (label27);
  gtk_table_attach (GTK_TABLE (table1), label27, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);

  label29 = gtk_label_new ("Referred to");
  gtk_widget_show (label29);
  gtk_table_attach (GTK_TABLE (table1), label29, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label29), 0, 0.5);

  label30 = gtk_label_new ("Project");
  gtk_widget_show (label30);
  gtk_table_attach (GTK_TABLE (table1), label30, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label30), 0, 0.5);

  label31 = gtk_label_new ("Date modified");
  gtk_widget_show (label31);
  gtk_table_attach (GTK_TABLE (table1), label31, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label31), 0, 0.5);

  label32 = gtk_label_new ("Date created");
  gtk_widget_show (label32);
  gtk_table_attach (GTK_TABLE (table1), label32, 0, 1, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label32), 0, 0.5);

  label33 = gtk_label_new ("Created by");
  gtk_widget_show (label33);
  gtk_table_attach (GTK_TABLE (table1), label33, 0, 1, 8, 9,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label33), 0, 0.5);

  combo_refs = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_refs)->popwin),
                     "GladeParentKey", combo_refs);
  gtk_widget_show (combo_refs);
  gtk_table_attach (GTK_TABLE (table1), combo_refs, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_refs = GTK_COMBO (combo_refs)->entry;
  gtk_widget_show (combo_entry_refs);

  combo_text = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_text)->popwin),
                     "GladeParentKey", combo_text);
  gtk_widget_show (combo_text);
  gtk_table_attach (GTK_TABLE (table1), combo_text, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_text = GTK_COMBO (combo_text)->entry;
  gtk_widget_show (combo_entry_text);

  combo_referred = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_referred)->popwin),
                     "GladeParentKey", combo_referred);
  gtk_widget_show (combo_referred);
  gtk_table_attach (GTK_TABLE (table1), combo_referred, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_referred = GTK_COMBO (combo_referred)->entry;
  gtk_widget_show (combo_entry_referred);

  combo_project = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_project)->popwin),
                     "GladeParentKey", combo_project);
  gtk_widget_show (combo_project);
  gtk_table_attach (GTK_TABLE (table1), combo_project, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_project = GTK_COMBO (combo_project)->entry;
  gtk_widget_show (combo_entry_project);

  combo_datemodified = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_datemodified)->popwin),
                     "GladeParentKey", combo_datemodified);
  gtk_widget_show (combo_datemodified);
  gtk_table_attach (GTK_TABLE (table1), combo_datemodified, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_datemodified = GTK_COMBO (combo_datemodified)->entry;
  gtk_widget_show (combo_entry_datemodified);

  combo_datecreated = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_datecreated)->popwin),
                     "GladeParentKey", combo_datecreated);
  gtk_widget_show (combo_datecreated);
  gtk_table_attach (GTK_TABLE (table1), combo_datecreated, 1, 2, 7, 8,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_datecreated = GTK_COMBO (combo_datecreated)->entry;
  gtk_widget_show (combo_entry_datecreated);

  combo_createdby = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_createdby)->popwin),
                     "GladeParentKey", combo_createdby);
  gtk_widget_show (combo_createdby);
  gtk_table_attach (GTK_TABLE (table1), combo_createdby, 1, 2, 8, 9,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry_createdby = GTK_COMBO (combo_createdby)->entry;
  gtk_widget_show (combo_entry_createdby);

  label16 = gtk_label_new ("label16");
  gtk_widget_show (label16);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 5), label16);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (notebook), vbox8);

  label23 = gtk_label_new ("All preparation is now ready.\n\nPress Apply to actually import the notes.");
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (vbox8), label23, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

  label22 = gtk_label_new ("label22");
  gtk_widget_show (label22);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 6), label22);

  hbox1 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 4);

  backbutton = gtk_button_new_from_stock ("gtk-go-back");
  gtk_widget_show (backbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), backbutton, FALSE, FALSE, 0);

  forwardbutton = gtk_button_new_from_stock ("gtk-go-forward");
  gtk_widget_show (forwardbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), forwardbutton, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (forwardbutton, GTK_CAN_DEFAULT);

  dialog_action_area1 = GTK_DIALOG (importnotesdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (importnotesdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  applybutton = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (importnotesdialog), applybutton, GTK_RESPONSE_APPLY);
  gtk_widget_set_sensitive (applybutton, FALSE);
  GTK_WIDGET_SET_FLAGS (applybutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) select_directory_button, "clicked",
                    G_CALLBACK (on_select_file_button_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) backbutton, "clicked",
                    G_CALLBACK (on_backbutton_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) forwardbutton, "clicked",
                    G_CALLBACK (on_forwardbutton_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) applybutton, "clicked",
                    G_CALLBACK (on_applybutton_clicked),
                    gpointer(this));

  gtk_widget_grab_focus (forwardbutton);
  gtk_widget_grab_default (forwardbutton);

  set_gui();
}


ImportNotesDialog::~ImportNotesDialog ()
{
  // Clean up temporary file.
  unlink (temporary_file (TEMP_FILE).c_str());
  gtk_widget_destroy (importnotesdialog);
}


int ImportNotesDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (importnotesdialog));
}


void ImportNotesDialog::on_backbutton_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportNotesDialog *) user_data)->on_backbutton ();
}


void ImportNotesDialog::on_backbutton () 
{
  gtk_notebook_prev_page (GTK_NOTEBOOK (notebook));
  set_gui();
}


void ImportNotesDialog::on_forwardbutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportNotesDialog *) user_data)->on_forwardbutton ();
}


void ImportNotesDialog::on_forwardbutton () 
{
  gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
  set_gui();
}


void ImportNotesDialog::on_select_file_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportNotesDialog *) user_data)->on_select_file_button ();
}


void ImportNotesDialog::on_select_file_button ()
{
  GtkWidget *file_selector;
  file_selector = gtk_file_selection_new ("Please choose a file");
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), file_to_import.c_str ());
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  string s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  if (result == GTK_RESPONSE_OK) {
    file_to_import = s;
    select_file_page();
  }
}


void ImportNotesDialog::set_gui()
{
  gtk_widget_set_sensitive (backbutton, TRUE);
  gtk_widget_set_sensitive (forwardbutton, TRUE);
  gtk_widget_set_sensitive (applybutton, FALSE);
  gint currentpage = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
  switch (currentpage)
  {
  case 0 :
    {
      gtk_widget_set_sensitive (backbutton, FALSE);
      break;
    }
  case 1 :
    {
      select_file_page();
      break;
    }
  case 2 :
    {
      // Nothing to do, text is already in the notebook.
      break;
    }
  case 3 :
    {
      unicode_page();
      break;
    }
  case 4 :
    {
      // Connect element that surrounds a full note.
      // Some eye candy, because the process might take time.
      WaitDialog wd (500, 450, 300);
      wd.run ();
      // Make known corrections to the xml file.
      make_known_corrections_to_xml_file ();
      // Dig up all elements in the xml file.
      dig_up_all_elements ();
      // Gui of page.
      begin_end_elements_page ();
      break;
    }
  case 5 :
    {
      content_elements_page ();
      break;
    }
  case 6 :
    {
      final_page ();
      gtk_widget_set_sensitive (forwardbutton, FALSE);
      gtk_widget_set_sensitive (applybutton, TRUE);
      break;
    }
  }
}


void ImportNotesDialog::select_file_page()
{
  // Show filename.
  gtk_label_set_text (GTK_LABEL (label8), file_to_import.c_str());
  // See whether the file seems an xml file.
  bool file_seems_xml = false;
  if (!file_to_import.empty()) {
    FILE *stream;
    string s = "head '";
    s.append (file_to_import);
    s.append ("'");
    stream = popen (s.c_str (), "r");
    char buf[10240];
    while (fgets (buf, sizeof (buf), stream)) {
      s = buf;
      if (s.find ("xml") != string::npos)
        file_seems_xml = true;
    }
    pclose (stream);
    // Info for user.
    if (file_seems_xml)
      gtk_label_set_text (GTK_LABEL (xml_file_label), "Fine, this file seems an xml file");
    else
      gtk_label_set_text (GTK_LABEL (xml_file_label), "This is not an xml file");
  }
  // We only can go forward if we've found an xml file.
  gtk_widget_set_sensitive (forwardbutton, file_seems_xml);
  // Copy the xml file to the temporary location.
  if (file_seems_xml)
  {
    string s = "cp '";
    s.append (file_to_import);
    s.append ("'");
    s.append (" ");
    s.append (temporary_file (TEMP_FILE));
    system(s.c_str());
  }
}


void ImportNotesDialog::unicode_page()
{
  // Some visual candy, because the process might take time.
  WaitDialog wd (500, 450, 300);
  wd.run ();
  // Filename of the file to import.
  string path = temporary_file (TEMP_FILE);
  // See whether the file contents is proper Unicode
  string file_contents;
  bool unicode_valid = true;
  {
    gchar *s;
    if (!g_file_get_contents (path.c_str(), &s, NULL, NULL))
      unicode_valid = false;
    file_contents = s;
    g_free (s);
  }
  ustring unicode_file_contents;
  unicode_file_contents = file_contents;
  if (!unicode_file_contents.validate ())
    unicode_valid = false;
  if (!unicode_valid) {
    // Unicode is not valid: Let the user convert it to proper Unicode, and then proceed.
    int dialog_result = GTK_RESPONSE_NONE;
    // Dialog should keep coming on until the OK response is given.
    // This is a work-around for a but in the delete-event in a dialog.
    // See the UnicodeDialog for more information about it.
    while (dialog_result != GTK_RESPONSE_OK) {
      UnicodeDialog unicodedialog (path);
      dialog_result = unicodedialog.run ();
    }
  }
}


void ImportNotesDialog::make_known_corrections_to_xml_file ()
{
  // Make a number of corrections to the xml file.
  string filename = temporary_file (TEMP_FILE);
  ReadText rt (filename, true);
  for (unsigned int i = 0; i < rt.lines.size(); i++) {
    // Get the line.
    string line = rt.lines[i];
    // The Glib xml parser gives an error on lines like:
    // ﻿<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
    // So remove that line.
    if (line.find ("xml") != string::npos)
      line = "";
    // Store the line again in the vector.
    rt.lines[i] = line;   
  }
  // Write the corrected file back to disk.
  write_lines (filename, rt.lines);
}


void ImportNotesDialog::dig_up_all_elements()
  // This lists all the different elements in the xml file.
{
  // Clear possible elements.
  elements_temp.clear();
  available_elements.clear();
  // Error variables
  bool was_error = false;
  string error_message;
  // Get contents of file.
  string filename = temporary_file (TEMP_FILE);
  gchar *contents;
  gsize  length;
  GError *error;
  error = NULL;
  if (!g_file_get_contents (filename.c_str(), &contents, &length, &error)) {
    cerr << error->message << endl;
    g_error_free (error);
    was_error = true;
    error_message = "Error reading xml file";
    return;
  }
  // Set up parser.
  GMarkupParseContext *context;
  GMarkupParser parser = {
    start_element_handler1,
    end_element_handler1,
    text_handler1,
    passthrough_handler,
    error_handler
  };
  // Parse xml file.
  if (!was_error) {
    context = g_markup_parse_context_new (&parser, GMarkupParseFlags (0), gpointer (this), NULL);
    if (!g_markup_parse_context_parse (context, contents, length, NULL)) {
      g_markup_parse_context_free (context);
      was_error = true;
      error_message = "Error parsing the xml file";
    }
  }
  if (!was_error) {
    if (!g_markup_parse_context_end_parse (context, NULL)) {
      g_markup_parse_context_free (context);
      was_error = true;
      error_message = "Error finishing to parse the xml file";
    }
  }
  if (!was_error)
    g_markup_parse_context_free (context);
  // Can we go forward? 
  gtk_widget_set_sensitive (forwardbutton, !was_error);
  // Display message?
  if (was_error) {
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (importnotesdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error_message.c_str ());
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
  // Store the different elements in the object.
  available_elements.assign (elements_temp.begin(), elements_temp.end());
}


void ImportNotesDialog::begin_end_elements_page ()
{
  // Fill the combobox with the elements.
  GList *glist = NULL;
  for (unsigned int i = 0; i < available_elements.size(); i++) {
    glist = g_list_append (glist, (gpointer) available_elements[i].c_str());
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_startelement), glist);
  g_list_free (glist);
  // Suggest value to start a note.
  for (unsigned int i = 0; i < available_elements.size(); i++)
    if (available_elements[i] == "Note")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_startelement), available_elements[i].c_str());
}


void ImportNotesDialog::content_elements_page ()
{
  // First save data from previous screen.
  note_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_startelement));
  // Fill the comboboxes with the elements.
  GList *glist = NULL;
  string not_available = "Element not available";
  glist = g_list_append (glist, (gpointer) not_available.c_str());
  for (unsigned int i = 0; i < available_elements.size(); i++) {
    glist = g_list_append (glist, (gpointer) available_elements[i].c_str());
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_refs), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_text), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_referred), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_project), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_datemodified), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_datecreated), glist);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_createdby), glist);
  g_list_free (glist);
  // Suggest values for the several elements.
  for (unsigned int i = 0; i < available_elements.size(); i++) {
    if (available_elements[i] == "References")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_refs), available_elements[i].c_str());
    if (available_elements[i] == "Contents")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_text), available_elements[i].c_str());
    if (available_elements[i] == "ReferredTo")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_referred), available_elements[i].c_str());
    if (available_elements[i] == "Project")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_project), available_elements[i].c_str());
    if (available_elements[i] == "ModificationDate")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_datemodified), available_elements[i].c_str());
    if (available_elements[i] == "CreationDate")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_datecreated), available_elements[i].c_str());
    if (available_elements[i] == "CreatedBy")
      gtk_entry_set_text (GTK_ENTRY (combo_entry_createdby), available_elements[i].c_str());
  }
}


void ImportNotesDialog::final_page ()
{
  // Save the values for the elements.
  reference_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_refs));
  notetext_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_text));
  referredto_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_referred));
  project_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_project));
  datemodified_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_datemodified));
  datecreated_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_datecreated));
  createdby_element = gtk_entry_get_text (GTK_ENTRY (combo_entry_createdby));
}


void ImportNotesDialog::on_applybutton_clicked (GtkButton *button, gpointer user_data)
{
  ((ImportNotesDialog *) user_data)->on_apply();
}


void ImportNotesDialog::on_apply()
/*
Parse the xml file.
The various handlers will then store the data in the database.
*/
{
  // Some error handling.
  bool was_error = false;
  string error_message;
  // Initialisation of depth control variables.
  depth = 0;
  depth_of_notetext = 100;
  // Get contents of file.
  string filename = temporary_file (TEMP_FILE);
  gchar *contents;
  gsize  length;
  GError *error;
  error = NULL;
  if (!g_file_get_contents (filename.c_str(), &contents, &length, &error)) {
    cerr << error->message << endl;
    g_error_free (error);
    was_error = true;
    error_message = "Error reading xml file";
    return;
  }
  // Set up parser.
  GMarkupParseContext *context;
  GMarkupParser parser = {
    start_element_handler2,
    end_element_handler2,
    text_handler2,
    passthrough_handler,
    error_handler
  };
  // Parse xml file.
  if (!was_error) {
    context = g_markup_parse_context_new (&parser, GMarkupParseFlags (0), gpointer (this), NULL);
    if (!g_markup_parse_context_parse (context, contents, length, NULL)) {
      g_markup_parse_context_free (context);
      was_error = true;
      error_message = "Error parsing the xml file";
    }
  }
  if (!was_error) {
    if (!g_markup_parse_context_end_parse (context, NULL)) {
      g_markup_parse_context_free (context);
      was_error = true;
      error_message = "Error finishing to parse the xml file";
    }
  }
  if (!was_error)
    g_markup_parse_context_free (context);
  // Display message?
  if (was_error) {
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (importnotesdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error_message.c_str ());
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
}


void ImportNotesDialog::start_element_handler1 (GMarkupParseContext *context,
                                                const gchar         *element_name,
                                                const gchar        **attribute_names,
                                                const gchar        **attribute_values,
                                                gpointer             user_data,
                                                GError             **error)
{
  ((ImportNotesDialog *) user_data)->elements_temp.insert (element_name);
/*
  int i;
  printf ("ELEMENT '%s'\n", element_name);
  i = 0;
  while (attribute_names[i] != NULL)
    {
      printf ("%s=\"%s\"\n", attribute_names[i], attribute_values[i]);
      ++i;
    }
*/
}


void ImportNotesDialog::start_element_handler2 (GMarkupParseContext *context,
                                                const gchar         *element_name,
                                                const gchar        **attribute_names,
                                                const gchar        **attribute_values,
                                                gpointer             user_data,
                                                GError             **error)
{
  string element;
  element = element_name;
  ((ImportNotesDialog *) user_data)->start_element_handler2 (element);
}


void ImportNotesDialog::end_element_handler1   (GMarkupParseContext *context,
                                                const gchar         *element_name,
                                                gpointer             user_data,
                                                GError             **error)
{
  // printf ("END '%s'\n", element_name);
}


void ImportNotesDialog::end_element_handler2   (GMarkupParseContext *context,
                                                const gchar         *element_name,
                                                gpointer             user_data,
                                                GError             **error)
{
  string element;
  element = element_name;
  ((ImportNotesDialog *) user_data)->end_element_handler2 (element);
}


void ImportNotesDialog::text_handler1          (GMarkupParseContext *context,
                                                const gchar         *text,
                                                gsize                text_len,
                                                gpointer             user_data,
                                                GError             **error)
{
  // printf ("TEXT '%.*s'\n", (int)text_len, text);
}



void ImportNotesDialog::text_handler2          (GMarkupParseContext *context,
                                                const gchar         *text,
                                                gsize                text_len,
                                                gpointer             user_data,
                                                GError             **error)
{
  ustring utext;
  utext = text;
  ((ImportNotesDialog *) user_data)->text_handler2 (utext);
}



void ImportNotesDialog::passthrough_handler    (GMarkupParseContext *context,
                                                const gchar         *passthrough_text,
                                                gsize                text_len,
                                                gpointer             user_data,
                                                GError             **error)
{
  // printf ("PASS '%.*s'\n", (int)text_len, passthrough_text);
}


void ImportNotesDialog::error_handler          (GMarkupParseContext *context,
                                                GError              *error,
                                                gpointer             user_data)
{
  cerr << error->message << endl;
}


void ImportNotesDialog::start_element_handler2 (const string& element_name)
/*
When we encounter a new element that starts data, this handler deals with that.
*/
{
  // Deal with depth of the elements.
  depth++;
  // Store depth of the note, so that we can catch everything nested deeper,
  // and treat it as note text also.
  if (element_name == notetext_element)
    depth_of_notetext = depth;
  if (element_name == note_element) {
    // New note starts from now. Clear storage area for new data.
    reference_content.clear();
    notetext_content.clear();
    status_content.clear();
    referredto_content.clear();
    project_content.clear();
    datemodified_content.clear();
    datecreated_content.clear();
    createdby_content.clear();
  }
  // Store element currently open.
  current_element = element_name;
}


void ImportNotesDialog::end_element_handler2   (const string& element_name)
/*
When we encounter an element that ends data, this handler deals with that.
*/
{
  // If new paragraph, deal with that.
  if (current_element == "P")
    notetext_content.append ("\n");
  if (element_name == note_element) {
    // We are at the end of a note. Go through the data, and fill up 
    // missing bits and pieces, and then store a new note in the database.

    // Right, let's get started. First the reference.
    {
      // There may be more than one reference, so deal with each of them.
      ParseLine parse (reference_content);
      reference_content.clear();
      for (unsigned int i = 0; i < parse.lines.size(); i++) {
        // Get the reference.
        string book, chapter, verse;
        if (reference_discover ("", "", "", trim (parse.lines[i]), book, chapter, verse))
        {
          string ref = book + " " + chapter + ":" + verse;
          ref = reference_to_numerical_equivalent (book, chapter, verse);
          if (!reference_content.empty())
            reference_content.append (" ");
          reference_content.append (ref);
        }
      }
      // If we did not find any reference, take Genesis 0:0.
      if (reference_content.empty())
        reference_content = "000000000";
    }

    // Deal with the text of the note.
    notetext_content = trim (notetext_content);
    
    // Deal with the "referred to" status of the note.
    int note_referredto;
    referredto_content = trim (referredto_content);
    note_referredto = notes_referred_to_index (referredto_content);

    // Deal with the project name.
    project_content = trim (project_content);

    // Deal with the date modified.
    int julian_modified;
    julian_modified = date_time_julian_day_get_parse (datemodified_content);

    // Deal with the date created.
    int julian_created;
    julian_created = date_time_julian_day_get_parse (datecreated_content);

    // Deal with the user.
    createdby_content = trim (createdby_content);

    // We've cleaned and verified all data. Store this note in the database.
    try {

      // Connect to database and start transaction
      sqlite3::connection con (notes_database_filename ().c_str ());
      con.executenonquery ("begin;");
      // ID (integer)
      gint unique_id;
      unique_id = notes_database_get_unique_id ();
      // References (text)
      // Project (text)
      // Status (integer)
      // Referred (integer)
      // Note (text) Apostrophies need to be doubled before storing them.
      notetext_content = double_apostrophy (notetext_content);
      // Casefolded (text)
      ustring casefolded = notetext_content.casefold();
      // Date created.
      // Date modified.
      // Username.
      // Insert data in database.
      con.executenonquery ("insert into %s values (%d, '%s', '%s', %d, %d, '%s', '%s', %d, %d, '%s');", TABLE_NOTES, unique_id, reference_content.c_str(), project_content.c_str(), 0, note_referredto, notetext_content.c_str(), casefolded.c_str(), julian_created, julian_modified, createdby_content.c_str());
      // Commit the transaction and close connection.
      con.executenonquery ("commit;");
      con.close ();
    }
    catch (exception & ex)
    {
      cerr << "Importing note: " << ex.what () << endl;
    }
  }    
  // Deal with depth of the elements.
  depth--;
}


void ImportNotesDialog::text_handler2          (const ustring& text)
/*
When we encounter the text inside an element, this handler deals with that.
*/
{
  if (current_element == reference_element) {
    reference_content.append(text);
  } else if (current_element == notetext_element) {
    notetext_content.append(text);
  } else if (current_element == referredto_element) {
    referredto_content.append(text);
  } else if (current_element == project_element) {
    project_content.append(text);
  } else if (current_element == datemodified_element) {
    datemodified_content.append(text);
  } else if (current_element == datecreated_element) {
    datecreated_content.append(text);
  } else if (current_element == createdby_element) {
    createdby_content.append(text);
  }
  if (depth > depth_of_notetext)
    notetext_content.append(text);
}
