/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogopen.h"
#include "projects.h"
#include "projectutils.h"
#include "directories.h"


OpenDialog::OpenDialog (const string & currentproject)
{
  // Get all projects
  Projects projects (directories_get_projects ());

  opendialog = gtk_dialog_new ();
  // Next one added to Glade's code.
  gtk_window_set_title (GTK_WINDOW (opendialog), "Open project");
  gtk_window_set_position (GTK_WINDOW (opendialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (opendialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (opendialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Select the project to open and then press OK");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 4);

  // Show it if there are no projects.
  if (projects.projects.size () == 0)
    {
      gtk_label_set_text (GTK_LABEL (label1), "There are no projects to open");
    }

  combo1 = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo1)->popwin), "GladeParentKey", combo1);
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox1), combo1, FALSE, FALSE, 4);
  // Produced by glade, but moved here.
  combo1_items = NULL;

  // Fill the combo box 
  for (unsigned int i = 0; i < projects.projects.size (); i++)
    {
      combo1_items = g_list_append (combo1_items, (gpointer) projects.projects[i].c_str ());
    }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);

  combo_entry1 = GTK_COMBO (combo1)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
  // Set the project now opened.
  if (currentproject.size ())
    gtk_entry_set_text (GTK_ENTRY (combo_entry1), currentproject.c_str ());
  gtk_entry_set_activates_default (GTK_ENTRY (combo_entry1), TRUE);

  dialog_action_area1 = GTK_DIALOG (opendialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (opendialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (opendialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  // Disable the OK button if there are no projects.
  if (projects.projects.size () == 0)
    gtk_widget_set_sensitive (okbutton, FALSE);

  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (static_on_okbutton_clicked), gpointer (this));

  gtk_widget_grab_focus (combo_entry1);
  gtk_widget_grab_default (okbutton);
}


OpenDialog::~OpenDialog ()
{
  gtk_widget_destroy (opendialog);
}


int
OpenDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (opendialog));
}


void
OpenDialog::static_on_okbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((OpenDialog *) user_data)->on_okbutton_clicked ();
}


void
OpenDialog::on_okbutton_clicked ()
{
  project = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
}
