/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogproject.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "dialogselectbooks.h"
#include "scripture.h"
#include "bible.h"
#include "directories.h"
#include <glib.h>
#include <config.h>


ProjectDialog::ProjectDialog (const string & projectname)
{
  // Save variabeles.
  currentprojectname = projectname;
  
  projectdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (projectdialog), "Project properties");
  gtk_window_set_position (GTK_WINDOW (projectdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (projectdialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (projectdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (projectdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Name:");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  nameentry = gtk_entry_new ();
  gtk_widget_show (nameentry);
  gtk_box_pack_start (GTK_BOX (vbox1), nameentry, FALSE, FALSE, 2);
  gtk_entry_set_text (GTK_ENTRY (nameentry), projectname.c_str ());
  gtk_entry_set_activates_default (GTK_ENTRY (nameentry), TRUE);
  // Set the name editable only in case of a new project.
  bool editable;
  editable = (projectname == NEW_PROJECT);
  gtk_editable_set_editable (GTK_EDITABLE (nameentry), editable);

  messagelabel = gtk_label_new ("");
  gtk_widget_show (messagelabel);
  gtk_box_pack_start (GTK_BOX (vbox1), messagelabel, FALSE, FALSE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 2);

  addbutton = gtk_button_new ();
  gtk_widget_show (addbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), addbutton, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (addbutton), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label5 = gtk_label_new_with_mnemonic ("Add book(s)");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 0);

  deletebutton = gtk_button_new ();
  gtk_widget_show (deletebutton);
  gtk_box_pack_start (GTK_BOX (hbox1), deletebutton, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (deletebutton), alignment2);

  hbox3 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox3);

  image2 = gtk_image_new_from_stock ("gtk-remove", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox3), image2, FALSE, FALSE, 0);

  label6 = gtk_label_new_with_mnemonic ("Delete book(s)");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox3), label6, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (projectdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (projectdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (projectdialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) nameentry, "changed",
                    G_CALLBACK (projectdialog_on_nameentry_changed), gpointer (this));
  g_signal_connect ((gpointer) addbutton, "clicked",
                    G_CALLBACK (projectdialog_on_addbutton_clicked), gpointer (this));
  g_signal_connect ((gpointer) deletebutton, "clicked",
                    G_CALLBACK (projectdialog_on_deletebutton_clicked), gpointer (this));
  g_signal_connect ((gpointer) cancelbutton1, "clicked",
                    G_CALLBACK (projectdialog_on_cancelbutton1_clicked), gpointer (this));
  g_signal_connect ((gpointer) okbutton1, "clicked",
                    G_CALLBACK (projectdialog_on_okbutton1_clicked), gpointer (this));

  gtk_widget_grab_focus (nameentry);
  gtk_widget_grab_default (okbutton1);

  // Handle gui.
  set_gui ();
}


ProjectDialog::~ProjectDialog ()
{
  gtk_widget_destroy (projectdialog);
}


int
ProjectDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (projectdialog));
}


bool
ProjectDialog::set_gui ()
{
  bool sensitive = true;
  newprojectname = gtk_entry_get_text (GTK_ENTRY (nameentry));
  newprojectname = trim (newprojectname);
  // Reject "New Project" as a name.
  if (newprojectname == NEW_PROJECT) {
    sensitive = false;
    gtk_label_set_text (GTK_LABEL (messagelabel), "(the name of the project should be changed)");
  }
  // Reject empty names.
  if (newprojectname.empty ()) {
    sensitive = false;
    gtk_label_set_text (GTK_LABEL (messagelabel), "(the project needs a name)");
  }
  // Reject names that already exist.
  if (currentprojectname != newprojectname)
    {
      Projects projects (directories_get_projects ());
      for (unsigned int i = 0; i < projects.projects.size (); i++)
        if (newprojectname == projects.projects[i])
          {
            sensitive = false;
            gtk_label_set_text (GTK_LABEL (messagelabel),
                                "(a project with this name already exists)");
          }
    }
  // Deal with the sensitivity of the dialog.
  gtk_widget_set_sensitive (okbutton1, sensitive);
  if (sensitive)
    gtk_label_set_text (GTK_LABEL (messagelabel), "");
  return true;
}


void
ProjectDialog::on_ok ()
{
  // Empty for just now.
}


void
ProjectDialog::on_cancel ()
{
  // Remove the "New Project", because it was created but not used..
  string delname = (g_build_filename (directories_get_projects ().c_str (), NEW_PROJECT, NULL));
  string command = "rm -rf '" + delname + "'";
  system (command.c_str ());
}


void
ProjectDialog::on_book_add ()
{
/*
 * This gives the user the possibility to add books that are still missing in 
 * the project. Only those books that are not yet in the project are shown.
 * If the user adds book(s), outlines of the book are placed in the project.
 * The outlines contains markers for chapters, verses, and the basics.
 * 
 * The outlines were taken from a text from BibleWorks, which was exported.
 * This exported text was converted by Bibledit Windows Companion to Paratext
 * format. Only the USFMs were written. Then with the Unix "rename" command
 * the names were changed, so that e.g. MAT.sfc became MAT.English.
 */

  vector < string > selectables;
  set < string > selection;
  Scripture scripture (currentprojectname);
  set < string > currentbooks;
  for (unsigned int i = 0; i < scripture.books.size (); i++)
    currentbooks.insert (scripture.books[i]);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
    {
      string book = index_to_biblebook (i);
      if (currentbooks.find (book) == currentbooks.end ())
        selectables.push_back (book);
    }
  SelectBooksDialog dialog (sbAdd, selectables, selection);
  int result = dialog.run ();
  if (result == GTK_RESPONSE_OK)
    {
      for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
        {
          string book = index_to_biblebook (i);
          if (!(selection.find (book) == selection.end ()))
            {
              string filename = projectutils_scripture_filename (currentprojectname, book);
              create_directory (g_path_get_dirname (filename.c_str ()));
              string s = index_to_paratext_id (i) + ".English";
              string templatename = g_build_filename (PACKAGE_DATA_DIR, s.c_str (), NULL);
              string command = "cp '" + templatename + "' '" + filename + "'";
              system (command.c_str ());
            }
        }
    }
}


void
ProjectDialog::on_book_delete ()
{
  vector < string > selectables;
  set < string > selection;
  Scripture scripture (currentprojectname);
  set < string > currentbooks;
  for (unsigned int i = 0; i < scripture.books.size (); i++)
    currentbooks.insert (scripture.books[i]);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
    {
      string book = index_to_biblebook (i);
      if (currentbooks.find (book) != currentbooks.end ())
        selectables.push_back (book);
    }
  SelectBooksDialog dialog (sbDelete, selectables, selection);
  int result = dialog.run ();
  if (result == GTK_RESPONSE_OK)
    {
      if (selection.empty ())
        return;
      GtkWidget *dialog1;
      dialog1 = gtk_message_dialog_new (GTK_WINDOW (projectdialog),
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_MESSAGE_QUESTION,
                                        GTK_BUTTONS_YES_NO,
                                        "Are you sure you want to delete the books?");
      int result = gtk_dialog_run (GTK_DIALOG (dialog1));
      gtk_widget_destroy (dialog1);
      if (result != GTK_RESPONSE_YES)
        return;
      GtkWidget *dialog2;
      dialog2 = gtk_message_dialog_new (GTK_WINDOW (projectdialog),
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_MESSAGE_QUESTION,
                                        GTK_BUTTONS_YES_NO,
                                        "Are you really sure to delete something worth perhaps months of work?");
      result = gtk_dialog_run (GTK_DIALOG (dialog2));
      gtk_widget_destroy (dialog2);
      if (result != GTK_RESPONSE_YES)
        return;
      for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
        {
          // Remove the book.
          string book = index_to_biblebook (i);
          if (selection.find (book) != selection.end ())
            {
              string filename = projectutils_scripture_filename (currentprojectname, book);
              string command = "rm -rf '";
              command.append (g_path_get_dirname (filename.c_str ()));
              command.append ("'");
              system (command.c_str ());
            }
        }
    }
}


void
ProjectDialog::projectdialog_on_nameentry_changed (GtkEditable * editable, gpointer user_data)
{
  ((ProjectDialog *) user_data)->set_gui ();
}


void
ProjectDialog::projectdialog_on_okbutton1_clicked (GtkButton * button, gpointer user_data)
{
  ((ProjectDialog *) user_data)->on_ok ();
}


void
ProjectDialog::projectdialog_on_addbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((ProjectDialog *) user_data)->on_book_add ();
}


void
ProjectDialog::projectdialog_on_deletebutton_clicked (GtkButton * button, gpointer user_data)
{
  ((ProjectDialog *) user_data)->on_book_delete ();
}


void
ProjectDialog::projectdialog_on_cancelbutton1_clicked (GtkButton * button, gpointer user_data)
{
  ((ProjectDialog *) user_data)->on_cancel ();
}
