/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogsearchspecial.h"
#include "utilities.h"
#include "sqlite3_plus.h"
#include "bible.h"
#include "worddatabase.h"
#include <set>
#include "scripture.h"
#include <glib/gthread.h>
#include "dialogprogress.h"
#include "referenceutils.h"
#include "references.h"


SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, GtkListStore * liststore)
{
  myconfiguration = configuration;
  Worddatabase worddatabase (configuration->project);
  mydatabase = worddatabase.dbfilename ();

  // Get the references from the editor.
  References references (liststore, NULL, NULL);
  references.get_loaded ();
  references.get_references (inputrefs);

  // Build dialog.
  searchspecialdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (searchspecialdialog), "Special Search");
  gtk_window_set_position (GTK_WINDOW (searchspecialdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (searchspecialdialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (searchspecialdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (searchspecialdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), notebook1, TRUE, TRUE, 0);

  vbox1 = gtk_vbox_new (FALSE, 8);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);

  label2 =
    gtk_label_new
    ("This looks through the references loaded in the editor\nto find out whether a certain word is found in it or not.");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  if (inputrefs.size ())
    {
      string s;
      s = convert_to_string (int (inputrefs.size ())) + " references loaded";
      word_per_verse_info_label = gtk_label_new (s.c_str ());
    }
  else
    {
      word_per_verse_info_label =
        gtk_label_new
        ("No references available.\nLoad references in the editor to complete this action.");
    }
  gtk_widget_show (word_per_verse_info_label);
  gtk_box_pack_start (GTK_BOX (vbox1), word_per_verse_info_label, FALSE, FALSE, 0);

  hbox1 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  label4 = gtk_label_new ("Word to search for:");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE, 0);

  word_per_verse_entry = gtk_entry_new ();
  gtk_widget_show (word_per_verse_entry);
  gtk_box_pack_start (GTK_BOX (hbox1), word_per_verse_entry, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (word_per_verse_entry, GTK_CAN_DEFAULT);
  gtk_entry_set_text (GTK_ENTRY (word_per_verse_entry), myconfiguration->find_searchword.c_str ());
  gtk_entry_set_activates_default (GTK_ENTRY (word_per_verse_entry), TRUE);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  radiobutton1_group = NULL;

  radiobutton1 =
    gtk_radio_button_new_with_mnemonic (NULL, "Look for references that do contain this word.");
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton1, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));

  radiobutton2 =
    gtk_radio_button_new_with_mnemonic (NULL, "Look for references that do not contain this word.");
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (vbox2), radiobutton2, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));

  if (configuration->searchspecial_containword)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton1), true);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton2), true);

  word_per_verse_checkbutton = gtk_check_button_new_with_mnemonic ("Case sensitive search");
  gtk_widget_show (word_per_verse_checkbutton);
  gtk_box_pack_start (GTK_BOX (vbox1), word_per_verse_checkbutton, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (word_per_verse_checkbutton),
                                myconfiguration->case_sensitive_search);

  if (inputrefs.size ())
    {
      word_per_verse_info_label2 =
        gtk_label_new
        ("Press Find to find all references that do not match\nand load them in the editor.");
    }
  else
    {
      word_per_verse_info_label2 = gtk_label_new ("");
    }
  gtk_widget_show (word_per_verse_info_label2);
  gtk_box_pack_start (GTK_BOX (vbox1), word_per_verse_info_label2, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (word_per_verse_info_label2), 0, 0.5);

  label1 = gtk_label_new ("Search in references");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
                              gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  dialog_action_area1 = GTK_DIALOG (searchspecialdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (searchspecialdialog), cancelbutton,
                                GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  findbutton = gtk_button_new_from_stock ("gtk-find");
  gtk_widget_show (findbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (searchspecialdialog), findbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (findbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) notebook1, "change_current_page",
                    G_CALLBACK (searchspecialdialog_on_notebook1_change_current_page), gpointer(this));
  g_signal_connect ((gpointer) word_per_verse_entry, "changed",
                    G_CALLBACK (searchspecialdialog_on_word_per_verse_entry_changed), gpointer(this));
  g_signal_connect ((gpointer) findbutton, "clicked",
                    G_CALLBACK (searchspecialdialog_on_findbutton_clicked), gpointer(this));

  gtk_widget_grab_focus (word_per_verse_entry);
  gtk_widget_grab_default (findbutton);

  set_gui ();
}


SearchSpecialDialog::~SearchSpecialDialog ()
{
  gtk_widget_destroy (searchspecialdialog);
}


int
SearchSpecialDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (searchspecialdialog));
}


void
SearchSpecialDialog::on_buttonfind_clicked ()
{
  thread_is_ready = false;
  percentage = 0;
  // Hide the dialog, so that the progress dialog becomes visible.
  gtk_widget_hide_all (searchspecialdialog);
  g_thread_create (GThreadFunc (searchspecialdialog_on_buttonfind_clicked_execute), gpointer(this), false, NULL);
  ProgressDialog dialog (&thread_is_ready, &percentage);
  dialog.run ();
}


void
SearchSpecialDialog::on_buttonfind_clicked_execute ()
{
  try
  {
    // Get and save some variables.
    ustring searchword = gtk_entry_get_text (GTK_ENTRY (word_per_verse_entry));
    myconfiguration->find_searchword = searchword;
    bool casesensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (word_per_verse_checkbutton));
    myconfiguration->case_sensitive_search = casesensitive;
    bool containsword = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton1));
    myconfiguration->searchspecial_containword = containsword;
    // Set a flag if the text is still being indexed at present.
    is_indexing = program_is_running ("bibledit.wdb");
    // Connect to database.
    sqlite3::connection con (mydatabase.c_str ());
    // Get the string to search for. Note any apostrophies need to be doubled for SQLite.
    // We need to normalize the search expression, as prescribed, when comparing strings.
    searchword = searchword.normalize ();
    if (!casesensitive)
      searchword = searchword.casefold ();
    searchword = double_apostrophy (searchword);
    vector <string> results_first_stage;
    // Execute the SQL statement, and read the results.
    sqlite3::reader reader;
    if (casesensitive)
      reader = con.executereader ("select book, chapter, verse from 'lines' where line glob ('*%s*');", searchword.c_str ());
    else
      reader = con.executereader ("select book, chapter, verse from 'lines' where casefolded glob ('*%s*');", searchword.c_str ());
    while (reader.read ())
      results_first_stage.push_back (book_chapter_verse_to_reference (reader.getint32 (0), reader.getint32 (1), reader.getint32 (2)));
    // Close reader.
    reader.close ();
    // Go through the list of references given to see if the word is in it.
    for (unsigned int i = 0; i < inputrefs.size (); i++)
      {
        percentage = double (i + 1) / double (inputrefs.size ());
        bool found = false;
        for (unsigned int i2 = 0; i2 < results_first_stage.size (); i2++)
          if (results_first_stage[i2] == inputrefs[i])
            {
              found = true;
              break;
            }
        if (found == containsword) results.push_back (inputrefs[i]);
      }
    // Close connection to db.
    con.close ();
  }
  catch (exception & ex)
  {
    cerr << "Error: " << ex.what () << endl;
  }
  thread_is_ready = true;
}


void
SearchSpecialDialog::set_gui ()
{
  bool b = false;
  if (inputrefs.size () > 0)
    {
      string searchword = gtk_entry_get_text (GTK_ENTRY (word_per_verse_entry));
      if (searchword.size () > 0)
        b = true;
    }
  gtk_widget_set_sensitive (findbutton, b);
}


void SearchSpecialDialog::searchspecialdialog_on_notebook1_change_current_page (GtkNotebook * notebook, gint offset,
                                                      gpointer user_data)
{
}


void SearchSpecialDialog::searchspecialdialog_on_word_per_verse_entry_changed (GtkEditable * editable, gpointer user_data)
{
  ((SearchSpecialDialog *) user_data)->set_gui ();
}


void SearchSpecialDialog::searchspecialdialog_on_findbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((SearchSpecialDialog *) user_data)->on_buttonfind_clicked ();
}


void SearchSpecialDialog::searchspecialdialog_on_buttonfind_clicked_execute (gpointer data)
{
  ((SearchSpecialDialog *) data)->on_buttonfind_clicked_execute ();
};
