/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "docbook_book.h"
#include <config.h>


DocbookBook::DocbookBook(const string& filename, const string& heading)
: writetext (filename)
{
  // XML header
  writetext.out << "<?xml version=\"1.0\" encoding=\"UTF8\"?>" << endl;
  // Reference to the local DTD
  writetext.out << "<!DOCTYPE article SYSTEM \"" << PACKAGE_DATA_DIR << "/docbookx.dtd\">" << endl;
  // Open the article.
  writetext.out << "<article>" << endl;
  // Write the article information.
  writetext.out << "<articleinfo>" << endl;
  writetext.out << "<title>" << heading << "</title>" << endl;
  writetext.out << "</articleinfo>" << endl;
  // Write a standard "About" section.
  writetext.out << "<sect1 id=\"about\">" << endl;
  writetext.out << "<title>About this Bible</title>" << endl;
  writetext.out << "<para>" << endl;
  writetext.out << "Produced by Bibledit" << endl;
  writetext.out << "</para>" << endl;
  writetext.out << "<para>" << endl;
  time_t value = time (0);
  string current_date = ctime (&value);
  current_date = trim (current_date);
  writetext.out << current_date << endl; // Output current date/time.
  writetext.out << "</para>" << endl;
  writetext.out << "</sect1>" << endl;
}


DocbookBook::~DocbookBook()
{
  // Close article.
  writetext.out << "</article>" << endl;
}


void DocbookBook::openbook (const string& id, const string& title)
{
  writetext.out << "<sect1 id=\"" << id << "\">" << endl;
  writetext.out << "<title>" << title << "</title>" << endl;
//  writetext.out << "<subtitle>Subtitle of the book</subtitle>" << endl;
}


void DocbookBook::closebook ()
{
  writetext.out << "</sect1>" << endl;
}


void DocbookBook::openchapter (const string& book, unsigned int chapternumber)
{
  writetext.out << "<sect2 id=\"" << book << chapternumber << "\">" << endl;
  writetext.out << "<title>" << chapternumber << "</title>" << endl;
}


void DocbookBook::closechapter ()
{
  writetext.out << "</sect2>" << endl;
}


void DocbookBook::openparagraph ()
{
  writetext.out << "<para>" << endl;
  
}


void DocbookBook::closeparagraph ()
{
  writetext.out << "</para>" << endl;
}


void DocbookBook::writeline (const string& line)
{
  writetext.out << line << endl;
}
