/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "mainwindow.h"
#include "libraries.h"
#include "dialogopen.h"
#include "dialogshowscript.h"
#include "constants.h"
#include "utilities.h"
#include "htmlbrowser.h"
#include "usfmtools.h"
#include "dialogfind.h"
#include "dialogreplace.h"
#include "log.h"
#include "dialogcompanion.h"
#include "dialoggotoreference.h"
#include <signal.h>
#include "bible.h"
#include "projectutils.h"
#include "dialogproject.h"
#include "dialogprojects.h"
#include "usfm.h"
#include "dialogimportsfm.h"
#include "worddatabase.h"
#include "dialogreplacing.h"
#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include "dialogsearchspecial.h"
#include "referenceutils.h"
#include <config.h>
#include "references.h"
#include "pdfviewer.h"
#include "chapter.h"
#include "dialogprintreferences.h"
#include "dialogprintproject.h"
#include "notes_utils.h"
#include "dialogfindnote.h"
#include "dialogimportnotes.h"
#include "sqlite3_plus.h"
#include "date_time_utils.h"
#include "dialognotes.h"
#include "dialogentry.h"
#include "projects.h"
#include "directories.h"
#include "dialogcompare.h"
#include "project.h"


/*
|
|
|
|
|
Construction and destruction
|
|
|
|
|
 */


MainWindow::MainWindow ():
track (0),
//backup (0), 
  undo (0), windowscompanion (true)
{
  // Entry in the log.
  log_message ("Bibledit starts");

  accel_group = gtk_accel_group_new ();

  mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  // Set titlebar.
  set_titlebar ("");
  gtk_window_set_default_size (GTK_WINDOW (mainwindow), 800, 600);
  gtk_window_set_default_icon_from_file (g_build_filename (PACKAGE_DATA_DIR, "icon.xpm", NULL),
                                         NULL);

  // Initialize configuration here, only after mainwindow has been initialized.
  configuration = new Configuration (GTK_WINDOW (mainwindow));

  // Size and position of the main window.
  gtk_window_resize (GTK_WINDOW (mainwindow), configuration->window_width,
                     configuration->window_height);
  gtk_window_move (GTK_WINDOW (mainwindow), configuration->window_x_position,
                   configuration->window_y_position);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainwindow), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 0);

  menuitem_file = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_widget_show (menuitem_file);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_file);

  menuitem_file_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_file), menuitem_file_menu);

  file_project = gtk_image_menu_item_new_with_mnemonic ("_Project");
  gtk_widget_show (file_project);
  gtk_container_add (GTK_CONTAINER (menuitem_file_menu), file_project);

  image463 = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image463);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_project), image463);

  file_project_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_project), file_project_menu);

  new1 = gtk_image_menu_item_new_with_mnemonic ("_New");
  gtk_widget_show (new1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), new1);

  image903 = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image903);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (new1), image903);

  open1 = gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
  gtk_widget_show (open1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), open1);

  close1 = gtk_image_menu_item_new_from_stock ("gtk-close", accel_group);
  gtk_widget_show (close1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), close1);

  delete1 = gtk_image_menu_item_new_from_stock ("gtk-delete", accel_group);
  gtk_widget_show (delete1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), delete1);

  print_project = gtk_image_menu_item_new_with_mnemonic ("_Print");
  gtk_widget_show (print_project);
  gtk_container_add (GTK_CONTAINER (file_project_menu), print_project);

  image805 = gtk_image_new_from_stock ("gtk-print", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image805);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (print_project), image805);

  properties1 = gtk_image_menu_item_new_from_stock ("gtk-properties", accel_group);
  gtk_widget_show (properties1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), properties1);

  import1 = gtk_image_menu_item_new_with_mnemonic ("_Import");
  gtk_widget_show (import1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), import1);

  image464 = gtk_image_new_from_stock ("gtk-convert", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image464);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (import1), image464);

  copy_project_to = gtk_image_menu_item_new_with_mnemonic ("_Copy to");
  gtk_widget_show (copy_project_to);
  gtk_container_add (GTK_CONTAINER (file_project_menu), copy_project_to);

  image2688 = gtk_image_new_from_stock ("gtk-copy", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image2688);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (copy_project_to), image2688);

  compare_with1 = gtk_image_menu_item_new_with_mnemonic ("_Compare with");
  gtk_widget_show (compare_with1);
  gtk_container_add (GTK_CONTAINER (file_project_menu), compare_with1);

  image2764 = gtk_image_new_from_stock ("gtk-zoom-fit", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image2764);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (compare_with1), image2764);

  file_references = gtk_image_menu_item_new_with_mnemonic ("_References");
  gtk_widget_show (file_references);
  gtk_container_add (GTK_CONTAINER (menuitem_file_menu), file_references);

  image465 = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image465);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_references), image465);

  file_references_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_references), file_references_menu);

  open_references1 = gtk_image_menu_item_new_with_mnemonic ("_Open");
  gtk_widget_show (open_references1);
  gtk_container_add (GTK_CONTAINER (file_references_menu), open_references1);

  image466 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image466);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (open_references1), image466);

  references_save_as = gtk_image_menu_item_new_from_stock ("gtk-save-as", accel_group);
  gtk_widget_show (references_save_as);
  gtk_container_add (GTK_CONTAINER (file_references_menu), references_save_as);

  print_references = gtk_image_menu_item_new_with_mnemonic ("_Print");
  gtk_widget_show (print_references);
  gtk_container_add (GTK_CONTAINER (file_references_menu), print_references);
  gtk_widget_add_accelerator (print_references, "activate", accel_group,
                              GDK_P, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image759 = gtk_image_new_from_stock ("gtk-print", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image759);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (print_references), image759);

  close_references = gtk_image_menu_item_new_with_mnemonic ("_Close");
  gtk_widget_show (close_references);
  gtk_container_add (GTK_CONTAINER (file_references_menu), close_references);

  image468 = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image468);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (close_references), image468);

  delete_references = gtk_image_menu_item_new_with_mnemonic ("_Delete");
  gtk_widget_show (delete_references);
  gtk_container_add (GTK_CONTAINER (file_references_menu), delete_references);

  image469 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image469);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (delete_references), image469);

  notes2 = gtk_image_menu_item_new_with_mnemonic ("_Notes");
  gtk_widget_show (notes2);
  gtk_container_add (GTK_CONTAINER (menuitem_file_menu), notes2);

  image936 = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image936);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (notes2), image936);

  notes2_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (notes2), notes2_menu);

  new_note = gtk_image_menu_item_new_from_stock ("gtk-new", accel_group);
  gtk_widget_show (new_note);
  gtk_container_add (GTK_CONTAINER (notes2_menu), new_note);

  delete_note = gtk_image_menu_item_new_with_mnemonic ("_Delete");
  gtk_widget_show (delete_note);
  gtk_container_add (GTK_CONTAINER (notes2_menu), delete_note);

  image963 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image963);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (delete_note), image963);

  import_notes = gtk_image_menu_item_new_with_mnemonic ("_Import");
  gtk_widget_show (import_notes);
  gtk_container_add (GTK_CONTAINER (notes2_menu), import_notes);

  image1455 = gtk_image_new_from_stock ("gtk-convert", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1455);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (import_notes), image1455);

  quit1 = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
  gtk_widget_show (quit1);
  gtk_container_add (GTK_CONTAINER (menuitem_file_menu), quit1);

  menuitem_edit = gtk_menu_item_new_with_mnemonic ("_Edit");
  gtk_widget_show (menuitem_edit);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_edit);

  menuitem_edit_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_edit), menuitem_edit_menu);

  cut1 = gtk_image_menu_item_new_from_stock ("gtk-cut", accel_group);
  gtk_widget_show (cut1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), cut1);

  copy1 = gtk_image_menu_item_new_from_stock ("gtk-copy", accel_group);
  gtk_widget_show (copy1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), copy1);

  paste1 = gtk_image_menu_item_new_from_stock ("gtk-paste", accel_group);
  gtk_widget_show (paste1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), paste1);

  separator2 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), separator2);
  gtk_widget_set_sensitive (separator2, FALSE);

  undo1 = gtk_image_menu_item_new_with_mnemonic ("_Undo");
  gtk_widget_show (undo1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), undo1);
  gtk_widget_add_accelerator (undo1, "activate", accel_group, GDK_Z, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image295 = gtk_image_new_from_stock ("gtk-undo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image295);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (undo1), image295);

  redo1 = gtk_image_menu_item_new_with_mnemonic ("_Redo");
  gtk_widget_show (redo1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), redo1);
  gtk_widget_add_accelerator (redo1, "activate", accel_group, GDK_Z,
                              GdkModifierType (GDK_CONTROL_MASK | GDK_SHIFT_MASK),
                              GTK_ACCEL_VISIBLE);

  image296 = gtk_image_new_from_stock ("gtk-redo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image296);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (redo1), image296);

  separator4 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator4);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), separator4);
  gtk_widget_set_sensitive (separator4, FALSE);

  find1 = gtk_image_menu_item_new_from_stock ("gtk-find", accel_group);
  gtk_widget_show (find1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), find1);

  find_and_replace1 = gtk_image_menu_item_new_from_stock ("gtk-find-and-replace", accel_group);
  gtk_widget_show (find_and_replace1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), find_and_replace1);

  findspecial1 = gtk_image_menu_item_new_with_mnemonic ("Find _Special");
  gtk_widget_show (findspecial1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), findspecial1);

  image313 = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image313);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (findspecial1), image313);

  find_in_notes1 = gtk_image_menu_item_new_with_mnemonic ("Find in _Notes");
  gtk_widget_show (find_in_notes1);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), find_in_notes1);

  image1430 = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1430);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (find_in_notes1), image1430);

  separator7 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator7);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), separator7);
  gtk_widget_set_sensitive (separator7, FALSE);

  get_references_from_note = gtk_image_menu_item_new_with_mnemonic ("_Get references from note");
  gtk_widget_show (get_references_from_note);
  gtk_container_add (GTK_CONTAINER (menuitem_edit_menu), get_references_from_note);

  image3158 = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image3158);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (get_references_from_note), image3158);

  menuitem_view = gtk_menu_item_new_with_mnemonic ("_View");
  gtk_widget_show (menuitem_view);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_view);

  menuitem_view_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_view), menuitem_view_menu);

  font1 = gtk_image_menu_item_new_from_stock ("gtk-select-font", accel_group);
  gtk_widget_show (font1);
  gtk_container_add (GTK_CONTAINER (menuitem_view_menu), font1);

  separator5 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator5);
  gtk_container_add (GTK_CONTAINER (menuitem_view_menu), separator5);
  gtk_widget_set_sensitive (separator5, FALSE);

  viewnotes = gtk_image_menu_item_new_with_mnemonic ("_Notes");
  gtk_widget_show (viewnotes);
  gtk_container_add (GTK_CONTAINER (menuitem_view_menu), viewnotes);

  image2627 = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image2627);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (viewnotes), image2627);

  viewnotes_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (viewnotes), viewnotes_menu);

  view_notes_group1 = NULL;

  notes_for_one_verse1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "_Notes for current verse");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (notes_for_one_verse1));
  gtk_widget_show (notes_for_one_verse1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), notes_for_one_verse1);

  notes_for_all_verses1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "_All notes");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (notes_for_all_verses1));
  gtk_widget_show (notes_for_all_verses1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), notes_for_all_verses1);

  notes_edited_today1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "_Today's notes");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (notes_edited_today1));
  gtk_widget_show (notes_edited_today1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), notes_edited_today1);

  to_myself = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Myself");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_myself));
  gtk_widget_show (to_myself);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_myself);

  to_subteam = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Subteam");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_subteam));
  gtk_widget_show (to_subteam);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_subteam);

  to_team1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Team");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_team1));
  gtk_widget_show (to_team1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_team1);

  to_panel1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Panel");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_panel1));
  gtk_widget_show (to_panel1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_panel1);

  to_church1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Church");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_church1));
  gtk_widget_show (to_church1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_church1);

  to_consultant1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Consultant");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_consultant1));
  gtk_widget_show (to_consultant1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_consultant1);

  to_bible_society1 = gtk_radio_menu_item_new_with_mnemonic (view_notes_group1, "Notes referred to _Bible Society");
  view_notes_group1 = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (to_bible_society1));
  gtk_widget_show (to_bible_society1);
  gtk_container_add (GTK_CONTAINER (viewnotes_menu), to_bible_society1);

  separator6 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator6);
  gtk_container_add (GTK_CONTAINER (menuitem_view_menu), separator6);
  gtk_widget_set_sensitive (separator6, FALSE);

  more_info_on_current_note1 = gtk_image_menu_item_new_with_mnemonic ("_More info on current note");
  gtk_widget_show (more_info_on_current_note1);
  gtk_container_add (GTK_CONTAINER (menuitem_view_menu), more_info_on_current_note1);

  image1838 = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1838);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (more_info_on_current_note1), image1838);

  insert1 = gtk_menu_item_new_with_mnemonic ("_Insert");
  gtk_widget_show (insert1);
  gtk_container_add (GTK_CONTAINER (menubar1), insert1);

  insert1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (insert1), insert1_menu);

  standard_text_1 = gtk_image_menu_item_new_with_mnemonic ("Standard text _1");
  gtk_widget_show (standard_text_1);
  gtk_container_add (GTK_CONTAINER (insert1_menu), standard_text_1);
  gtk_widget_add_accelerator (standard_text_1, "activate", accel_group,
                              GDK_1, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1963 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1963);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (standard_text_1), image1963);

  standard_text_2 = gtk_image_menu_item_new_with_mnemonic ("Standard text _2");
  gtk_widget_show (standard_text_2);
  gtk_container_add (GTK_CONTAINER (insert1_menu), standard_text_2);
  gtk_widget_add_accelerator (standard_text_2, "activate", accel_group,
                              GDK_2, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1964 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1964);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (standard_text_2), image1964);

  standard_text_3 = gtk_image_menu_item_new_with_mnemonic ("Standard text _3");
  gtk_widget_show (standard_text_3);
  gtk_container_add (GTK_CONTAINER (insert1_menu), standard_text_3);
  gtk_widget_add_accelerator (standard_text_3, "activate", accel_group,
                              GDK_3, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1965 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1965);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (standard_text_3), image1965);

  standard_text_4 = gtk_image_menu_item_new_with_mnemonic ("Standard text _4");
  gtk_widget_show (standard_text_4);
  gtk_container_add (GTK_CONTAINER (insert1_menu), standard_text_4);
  gtk_widget_add_accelerator (standard_text_4, "activate", accel_group,
                              GDK_4, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1966 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1966);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (standard_text_4), image1966);

  menuitem_goto = gtk_menu_item_new_with_mnemonic ("_Goto");
  gtk_widget_show (menuitem_goto);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_goto);

  menuitem_goto_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_goto), menuitem_goto_menu);

  next_verse1 = gtk_image_menu_item_new_with_mnemonic ("Next verse");
  gtk_widget_show (next_verse1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), next_verse1);
  gtk_widget_add_accelerator (next_verse1, "activate", accel_group, GDK_F7, GdkModifierType (0),
                              GTK_ACCEL_VISIBLE);
  // The GdkModifierType was given as 0 by glade's code, but that did not compile.
  // Here we take a 0 and cast it to a GdkModifierType. That works well.

  image95 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image95);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_verse1), image95);

  previous_verse1 = gtk_image_menu_item_new_with_mnemonic ("Previous verse");
  gtk_widget_show (previous_verse1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), previous_verse1);
  gtk_widget_add_accelerator (previous_verse1, "activate", accel_group, GDK_F7, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  image96 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image96);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_verse1), image96);

  next_chapter1 = gtk_image_menu_item_new_with_mnemonic ("Next chapter");
  gtk_widget_show (next_chapter1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), next_chapter1);
  gtk_widget_add_accelerator (next_chapter1, "activate", accel_group, GDK_F8, GdkModifierType (0),
                              GTK_ACCEL_VISIBLE);

  image97 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image97);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_chapter1), image97);

  previous_chapter1 = gtk_image_menu_item_new_with_mnemonic ("Previous chapter");
  gtk_widget_show (previous_chapter1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), previous_chapter1);
  gtk_widget_add_accelerator (previous_chapter1, "activate", accel_group, GDK_F8, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  image98 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image98);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_chapter1), image98);

  next_book1 = gtk_image_menu_item_new_with_mnemonic ("Next book");
  gtk_widget_show (next_book1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), next_book1);
  gtk_widget_add_accelerator (next_book1, "activate", accel_group, GDK_F9, GdkModifierType (0),
                              GTK_ACCEL_VISIBLE);

  image99 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image99);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_book1), image99);

  previous_book1 = gtk_image_menu_item_new_with_mnemonic ("Previous book");
  gtk_widget_show (previous_book1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), previous_book1);
  gtk_widget_add_accelerator (previous_book1, "activate", accel_group, GDK_F9, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  image100 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image100);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_book1), image100);

  next_reference1 = gtk_image_menu_item_new_with_mnemonic ("Next reference");
  gtk_widget_show (next_reference1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), next_reference1);
  gtk_widget_add_accelerator (next_reference1, "activate", accel_group, GDK_F6, GdkModifierType (0),
                              GTK_ACCEL_VISIBLE);

  image608 = gtk_image_new_from_stock ("gtk-go-down", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image608);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_reference1), image608);

  previous_reference1 = gtk_image_menu_item_new_with_mnemonic ("Previous reference");
  gtk_widget_show (previous_reference1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), previous_reference1);
  gtk_widget_add_accelerator (previous_reference1, "activate", accel_group, GDK_F6, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  image609 = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image609);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_reference1), image609);

  reference1 = gtk_image_menu_item_new_with_mnemonic ("Reference");
  gtk_widget_show (reference1);
  gtk_container_add (GTK_CONTAINER (menuitem_goto_menu), reference1);
  gtk_widget_add_accelerator (reference1, "activate", accel_group, GDK_g, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image101 = gtk_image_new_from_stock ("gtk-jump-to", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image101);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (reference1), image101);

  menuitem_preferences = gtk_menu_item_new_with_mnemonic ("P_references");
  gtk_widget_show (menuitem_preferences);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_preferences);

  menuitem_preferences_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_preferences), menuitem_preferences_menu);

  bibledit_windows_companion1 =
    gtk_image_menu_item_new_with_mnemonic ("_Bibledit Windows Companion");
  gtk_widget_show (bibledit_windows_companion1);
  gtk_container_add (GTK_CONTAINER (menuitem_preferences_menu), bibledit_windows_companion1);

  image103 = gtk_image_new_from_stock ("gtk-network", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image103);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (bibledit_windows_companion1), image103);

  notes_preferences = gtk_image_menu_item_new_with_mnemonic ("_Notes");
  gtk_widget_show (notes_preferences);
  gtk_container_add (GTK_CONTAINER (menuitem_preferences_menu), notes_preferences);

  image2116 = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image2116);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (notes_preferences), image2116);

  menuitem_diagnostics = gtk_menu_item_new_with_mnemonic ("_Diagnostics");
  gtk_widget_show (menuitem_diagnostics);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_diagnostics);

  menuitem_diagnostics_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_diagnostics), menuitem_diagnostics_menu);

  view_system_log1 = gtk_image_menu_item_new_with_mnemonic ("_View system log");
  gtk_widget_show (view_system_log1);
  gtk_container_add (GTK_CONTAINER (menuitem_diagnostics_menu), view_system_log1);

  image106 = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image106);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (view_system_log1), image106);

  menuitem_help = gtk_menu_item_new_with_mnemonic ("_Help");
  gtk_widget_show (menuitem_help);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem_help);

  menuitem_help_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_help), menuitem_help_menu);

  help1 = gtk_image_menu_item_new_from_stock ("gtk-help", accel_group);
  gtk_widget_show (help1);
  gtk_container_add (GTK_CONTAINER (menuitem_help_menu), help1);

  about1 = gtk_image_menu_item_new_with_mnemonic ("_About");
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (menuitem_help_menu), about1);

  image107 = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image107);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (about1), image107);

  toolbar1 = gtk_toolbar_new ();
  gtk_widget_show (toolbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_BOTH);

  button_back = gtk_button_new ();
  gtk_widget_show (button_back);
  gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_CHILD_WIDGET, button_back, "",
                              NULL, NULL, NULL, NULL, NULL);

  image1 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_container_add (GTK_CONTAINER (button_back), image1);

  button_forward = gtk_button_new ();
  gtk_widget_show (button_forward);
  gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_CHILD_WIDGET, button_forward, "",
                              NULL, NULL, NULL, NULL, NULL);

  image2 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_container_add (GTK_CONTAINER (button_forward), image2);

  combo_book = gtk_combo_new ();
  gtk_widget_show (combo_book);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar1), combo_book, NULL, NULL);

  combo_entry_book = GTK_COMBO (combo_book)->entry;
  gtk_widget_show (combo_entry_book);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry_book), FALSE);

  combo_chapter = gtk_combo_new ();
  gtk_widget_show (combo_chapter);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar1), combo_chapter, NULL, NULL);

  combo_entry_chapter = GTK_COMBO (combo_chapter)->entry;
  gtk_widget_show (combo_entry_chapter);
  gtk_widget_set_size_request (combo_entry_chapter, 55, -1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry_chapter), FALSE);

  combo_verse = gtk_combo_new ();
  gtk_widget_show (combo_verse);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar1), combo_verse, NULL, NULL);

  combo_entry_verse = GTK_COMBO (combo_verse)->entry;
  gtk_widget_show (combo_entry_verse);
  gtk_widget_set_size_request (combo_entry_verse, 55, -1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry_verse), FALSE);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (hbox1), hpaned1, TRUE, TRUE, 0);
  // Position of horizontal pane.
  gtk_paned_set_position (GTK_PANED (hpaned1), configuration->hpane_position);

  vpaned1 = gtk_vpaned_new ();
  gtk_widget_show (vpaned1);
  gtk_paned_pack1 (GTK_PANED (hpaned1), vpaned1, FALSE, TRUE);
  // Position of vertical pane.
  gtk_paned_set_position (GTK_PANED (vpaned1), configuration->vpane_position);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_paned_pack1 (GTK_PANED (vpaned1), scrolledwindow1, FALSE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);

  textview_edit = gtk_text_view_new ();
  gtk_widget_show (textview_edit);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview_edit);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview_edit), GTK_WRAP_WORD);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_paned_pack2 (GTK_PANED (vpaned1), notebook1, TRUE, TRUE);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook1), FALSE);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  textview_notes = gtk_text_view_new ();
  gtk_widget_show (textview_notes);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), textview_notes);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview_notes), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview_notes), GTK_WRAP_WORD);

  label1 = gtk_label_new ("label1");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  hbox3 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (notebook1), hbox3);
  gtk_container_set_border_width (GTK_CONTAINER (hbox3), 1);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_box_pack_start (GTK_BOX (hbox3), scrolledwindow4, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  textview_note = gtk_text_view_new ();
  gtk_widget_show (textview_note);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), textview_note);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview_note), GTK_WRAP_WORD);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow5);
  gtk_box_pack_start (GTK_BOX (hbox3), scrolledwindow5, FALSE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), viewport1);

  vbox2 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (viewport1), vbox2);

  label5 = gtk_label_new ("Referred to");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (vbox2), label5, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  combo_ref_to = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_ref_to)->popwin),
                     "GladeParentKey", combo_ref_to);
  gtk_widget_show (combo_ref_to);
  gtk_box_pack_start (GTK_BOX (vbox2), combo_ref_to, FALSE, FALSE, 0);
  gtk_widget_set_size_request (combo_ref_to, 60, -1);

  combo_ref_to_entry = GTK_COMBO (combo_ref_to)->entry;
  gtk_widget_show (combo_ref_to_entry);
  gtk_editable_set_editable (GTK_EDITABLE (combo_ref_to_entry), FALSE);

  label3 = gtk_label_new ("References");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox2), label3, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow6, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  textview_refs = gtk_text_view_new ();
  gtk_widget_show (textview_refs);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), textview_refs);

  label6 = gtk_label_new ("Project");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  combo_project = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo_project)->popwin),
                     "GladeParentKey", combo_project);
  gtk_widget_show (combo_project);
  gtk_box_pack_start (GTK_BOX (vbox2), combo_project, FALSE, FALSE, 0);
  gtk_widget_set_size_request (combo_project, 60, -1);

  combo_project_entry = GTK_COMBO (combo_project)->entry;
  gtk_widget_show (combo_project_entry);
  gtk_editable_set_editable (GTK_EDITABLE (combo_project_entry), FALSE);

  hbox4 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, FALSE, FALSE, 0);

  button_cancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button_cancel);
  gtk_box_pack_start (GTK_BOX (hbox4), button_cancel, FALSE, FALSE, 0);
  gtk_widget_set_size_request (button_cancel, 75, -1);

  button_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button_ok);
  gtk_box_pack_start (GTK_BOX (hbox4), button_ok, FALSE, FALSE, 0);
  gtk_widget_set_size_request (button_ok, 75, -1);

  label2 = gtk_label_new ("label2");
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_paned_pack2 (GTK_PANED (hpaned1), scrolledwindow2, TRUE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  // Manually added and changed.
  liststore = gtk_list_store_new (1, G_TYPE_STRING);
  // Text cell renderer.
  GtkCellRenderer *renderer;
  renderer = gtk_cell_renderer_text_new ();
  treeview1 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (liststore));
  gtk_widget_show (treeview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), treeview1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview1), TRUE);
  treecolumn = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), treecolumn);
  treeselect = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
  gtk_tree_selection_set_mode (treeselect, GTK_SELECTION_MULTIPLE);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);

  // Appearance of text in editor.
  editor_set_font ();

  g_signal_connect ((gpointer) mainwindow, "destroy", G_CALLBACK (gtk_main_quit), gpointer(this));
  g_signal_connect ((gpointer) mainwindow, "delete_event", G_CALLBACK (on_mainwindow_delete_event), gpointer(this));
  g_signal_connect ((gpointer) new1, "activate", G_CALLBACK (on_new1_activate), gpointer(this));
  g_signal_connect ((gpointer) open1, "activate", G_CALLBACK (on_open1_activate), gpointer(this));
  g_signal_connect ((gpointer) close1, "activate", G_CALLBACK (on_close1_activate), gpointer(this));
  g_signal_connect ((gpointer) delete1, "activate", G_CALLBACK (on_delete1_activate), gpointer(this));
  g_signal_connect ((gpointer) print_project, "activate", G_CALLBACK (on_print_project_activate), gpointer(this));
  g_signal_connect ((gpointer) properties1, "activate", G_CALLBACK (on_properties1_activate), gpointer(this));
  g_signal_connect ((gpointer) import1, "activate", G_CALLBACK (on_import1_activate), gpointer(this));
  g_signal_connect ((gpointer) copy_project_to, "activate", G_CALLBACK (on_copy_project_to_activate), gpointer(this));
  g_signal_connect ((gpointer) compare_with1, "activate", G_CALLBACK (on_compare_with1_activate), gpointer(this));
  g_signal_connect ((gpointer) open_references1, "activate", G_CALLBACK (on_open_references1_activate), gpointer(this));
  g_signal_connect ((gpointer) references_save_as, "activate", G_CALLBACK (on_references_save_as_activate), gpointer(this));
  g_signal_connect ((gpointer) print_references, "activate", G_CALLBACK (on_print_references_activate), gpointer(this));
  g_signal_connect ((gpointer) close_references, "activate", G_CALLBACK (on_close_references_activate), gpointer (this));
  g_signal_connect ((gpointer) delete_references, "activate", G_CALLBACK (on_delete_references_activate), gpointer (this));
  g_signal_connect ((gpointer) new_note, "activate", G_CALLBACK (on_new_note_activate), gpointer(this));
  g_signal_connect ((gpointer) delete_note, "activate", G_CALLBACK (on_delete_note_activate), gpointer(this));
  g_signal_connect ((gpointer) import_notes, "activate", G_CALLBACK (on_import_notes_activate), gpointer(this));
  g_signal_connect ((gpointer) quit1, "activate", G_CALLBACK (on_quit1_activate), gpointer(this));
  g_signal_connect ((gpointer) menuitem_edit, "activate", G_CALLBACK (on_edit1_activate), gpointer(this));
  g_signal_connect ((gpointer) cut1, "activate", G_CALLBACK (on_cut1_activate), gpointer(this));
  g_signal_connect ((gpointer) copy1, "activate", G_CALLBACK (on_copy1_activate), gpointer(this));
  g_signal_connect ((gpointer) paste1, "activate", G_CALLBACK (on_paste1_activate), gpointer(this));
  g_signal_connect ((gpointer) undo1, "activate", G_CALLBACK (on_undo1_activate), gpointer(this));
  g_signal_connect ((gpointer) redo1, "activate", G_CALLBACK (on_redo1_activate), gpointer(this));
  g_signal_connect ((gpointer) find1, "activate", G_CALLBACK (on_find1_activate), gpointer(this));
  g_signal_connect ((gpointer) find_and_replace1, "activate", G_CALLBACK (on_find_and_replace1_activate), gpointer (this));
  g_signal_connect ((gpointer) findspecial1, "activate", G_CALLBACK (on_findspecial1_activate), gpointer(this));
  g_signal_connect ((gpointer) find_in_notes1, "activate", G_CALLBACK (on_find_in_notes1_activate), gpointer(this));
  g_signal_connect ((gpointer) get_references_from_note, "activate", G_CALLBACK (on_get_references_from_note_activate), gpointer(this));
  g_signal_connect ((gpointer) menuitem_view, "activate", G_CALLBACK (on_menuitem_view_activate), gpointer(this));
  g_signal_connect ((gpointer) font1, "activate", G_CALLBACK (on_font1_activate), gpointer(this));
  g_signal_connect ((gpointer) notes_for_one_verse1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) notes_for_all_verses1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) notes_edited_today1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_myself, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_subteam, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_team1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_panel1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_church1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_consultant1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) to_bible_society1, "activate", G_CALLBACK (on_notes_redisplay), gpointer(this));
  g_signal_connect ((gpointer) more_info_on_current_note1, "activate", G_CALLBACK (on_more_info_on_current_note1_activate), gpointer(this));
  g_signal_connect ((gpointer) insert1, "activate", G_CALLBACK (on_insert1_activate), gpointer(this));
  g_signal_connect ((gpointer) standard_text_1, "activate", G_CALLBACK (on_standard_text_1_activate), gpointer (this));
  g_signal_connect ((gpointer) standard_text_2, "activate", G_CALLBACK (on_standard_text_2_activate), gpointer (this));
  g_signal_connect ((gpointer) standard_text_3, "activate", G_CALLBACK (on_standard_text_3_activate), gpointer (this));
  g_signal_connect ((gpointer) standard_text_4, "activate", G_CALLBACK (on_standard_text_4_activate), gpointer (this));
  g_signal_connect ((gpointer) next_verse1, "activate", G_CALLBACK (on_next_verse1_activate), gpointer (this));
  g_signal_connect ((gpointer) previous_verse1, "activate", G_CALLBACK (on_previous_verse1_activate), gpointer (this));
  g_signal_connect ((gpointer) next_chapter1, "activate", G_CALLBACK (on_next_chapter1_activate), gpointer (this));
  g_signal_connect ((gpointer) previous_chapter1, "activate", G_CALLBACK (on_previous_chapter1_activate), gpointer (this));
  g_signal_connect ((gpointer) next_book1, "activate", G_CALLBACK (on_next_book1_activate), gpointer (this));
  g_signal_connect ((gpointer) previous_book1, "activate", G_CALLBACK (on_previous_book1_activate), gpointer (this));
  g_signal_connect ((gpointer) next_reference1, "activate", G_CALLBACK (on_next_reference1_activate), gpointer(this));
  g_signal_connect ((gpointer) previous_reference1, "activate", G_CALLBACK (on_previous_reference1_activate), gpointer(this));
  g_signal_connect ((gpointer) reference1, "activate", G_CALLBACK (on_reference1_activate), gpointer(this));
  g_signal_connect ((gpointer) bibledit_windows_companion1, "activate", G_CALLBACK (on_bibledit_windows_companion1_activate), gpointer(this));
  g_signal_connect ((gpointer) notes_preferences, "activate", G_CALLBACK (on_notes_preferences_activate), gpointer(this));
  g_signal_connect ((gpointer) view_system_log1, "activate", G_CALLBACK (on_view_system_log1_activate), gpointer(this));
  g_signal_connect ((gpointer) help1, "activate", G_CALLBACK (on_help1_activate), gpointer(this));
  g_signal_connect ((gpointer) about1, "activate", G_CALLBACK (on_about1_activate), gpointer(this));
  g_signal_connect ((gpointer) button_back, "clicked", G_CALLBACK (on_button_back_clicked), gpointer(this));
  g_signal_connect ((gpointer) button_forward, "clicked", G_CALLBACK (on_button_forward_clicked), gpointer(this));
  g_signal_connect ((gpointer) combo_entry_book, "changed", G_CALLBACK (on_combo_entry_book_changed), gpointer(this));
  g_signal_connect ((gpointer) combo_entry_chapter, "changed", G_CALLBACK (on_combo_entry_chapter_changed), gpointer(this));
  g_signal_connect ((gpointer) combo_entry_verse, "changed", G_CALLBACK (on_combo_entry_verse_changed), gpointer(this));
  g_signal_connect ((gpointer) textview_edit, "move_cursor", G_CALLBACK (on_textview_edit_move_cursor), gpointer(this));
  g_signal_connect ((gpointer) textview_edit, "grab_focus", G_CALLBACK (on_textview_edit_grab_focus), gpointer(this));
  g_signal_connect ((gpointer) button_cancel, "clicked", G_CALLBACK (on_button_cancel_clicked), gpointer(this));
  g_signal_connect ((gpointer) button_ok, "clicked", G_CALLBACK (on_button_ok_clicked), gpointer(this));
  g_signal_connect ((gpointer) treeview1, "key_press_event", G_CALLBACK (on_treeview1_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview1, "button_press_event", G_CALLBACK (on_treeview1_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) textview_notes, "motion-notify-event", G_CALLBACK (notes_motion_notify_event), gpointer(this));
  g_signal_connect ((gpointer) textview_notes, "visibility-notify-event", G_CALLBACK (notes_visibility_notify_event), gpointer(this));
  g_signal_connect ((gpointer) textview_notes, "key-press-event", G_CALLBACK (notes_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) textview_notes, "event-after", G_CALLBACK (notes_event_after), gpointer(this));
  g_signal_connect ((gpointer) combo_ref_to_entry, "changed", G_CALLBACK (on_combo_entry_ref_to_changed), gpointer (this));

  gtk_window_add_accel_group (GTK_WINDOW (mainwindow), accel_group);

  // Store pointer to the two text buffers to simplify later use.
  // The reference count on the buffer is not incremented; 
  // the caller of this function won't own a new reference.
  // Therefore the buffers do not need to be freed.
  textbuffer_edit = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_edit));
  textbuffer_notes = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_notes));

  // Signal handlers for the undo/redo function.
  g_signal_connect ((gpointer) textbuffer_edit, "insert_text", G_CALLBACK (on_textbuffer_edit_insert_text), gpointer(this));
  g_signal_connect ((gpointer) textbuffer_edit, "delete_range", G_CALLBACK (on_textbuffer_edit_delete_text), gpointer(this));

  // Tag for highlighting a new verse.
  // The colour names can be found in rgb.txt of the X system.
  // On Fedore Core 3 this is in /usr/X11R6/lib/X11/rgb.txt
  editor_line_tag = gtk_text_buffer_create_tag (textbuffer_edit, "linetag", "background", "light yellow", NULL);

  // Tag for highlighting search words
  editor_reference_tag = gtk_text_buffer_create_tag (textbuffer_edit, "referencetag", "background", "khaki", NULL);

  // Statusbar context id
  statusbar_context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar1), "Statusbar");

  // Initalize some variables.
  go_to_reference_running = false;
  scripture = NULL;
  navigator_book = NULL;
  book = NULL;

  // Project initialization.
  initialize_project ();

  // Get the system for tracking the Bible references up and running.
  g_timeout_add (1000, GSourceFunc (mainwindow_on_tracker_timeout), gpointer(this));
  set_sensitivity_of_navigator_buttons ();

  // System for navigation.
  g_timeout_add (100, GSourceFunc (mainwindow_on_navigation_timeout), gpointer(this));

  // Automatic saving of the file, periodiodically.
  g_timeout_add (60000, GSourceFunc (mainwindow_on_save_timeout), gpointer(this));

  // Communication with Windows Companion
  if (configuration->companion_enabled)
    windowscompanion.Connect ("localhost", 51515);
  g_timeout_add (100, GSourceFunc (mainwindow_on_windows_companion_timeout), gpointer(this));

  // Block the signal of a pipe error that otherwise would kill bibledit.
  signal (SIGPIPE, SIG_IGN);

  // Load previously saved references, if any.
  References references (liststore, treeview1, treecolumn);
  references.load ("");
  references.get_references (references_searchwords, references_casesensitives);
  references.fill_store ();

  // Initialize the notes view.
  hovering_over_link = FALSE;
  hand_cursor = NULL;
  regular_cursor = NULL;
  hand_cursor = gdk_cursor_new (GDK_HAND2);
  regular_cursor = gdk_cursor_new (GDK_XTERM);
  // Initialize the pointers to the text buffers.
  textbuffer_note = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note));
  textbuffer_references = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_refs));
  g_signal_connect ((gpointer) textbuffer_note, "changed", G_CALLBACK (on_note_editor_changed), gpointer(this));

  g_timeout_add (100, GSourceFunc (on_display_notes_timeout), gpointer(this));
  notes_redisplay ();
  
  // Finally: show main window.
  gtk_widget_show (mainwindow);
}


MainWindow::~MainWindow ()
{
  // Save the size and position of the program window.
  // The get_position function is broken, See the gtk documentation for reasons.
  // Anyway, for just now we'll leave the variables in configuration, and we hope
  // that it will be fixed later. The problem now is that the function will
  // return the value that was set with move(x,y). If the user moves the window
  // that is not reflected here, but it will keep returning the values set
  // with move(x,y).
  // If you want to change the default values when Bibledit starts, edit
  // the configuration file
  gtk_window_get_size (GTK_WINDOW (mainwindow), &configuration->window_width, &configuration->window_height);
  gtk_window_get_position (GTK_WINDOW (mainwindow), &configuration->window_x_position, &configuration->window_y_position);
  configuration->hpane_position = gtk_paned_get_position (GTK_PANED (hpaned1));
  configuration->vpane_position = gtk_paned_get_position (GTK_PANED (vpaned1));
  // Save text in editor
  editor_save ();
  // Remove system log.
  unlink (LOG_FILE);
  if (scripture)
    delete scripture;
  if (navigator_book)
    delete navigator_book;
  if (book)
    delete book;
  // Optimize word database
  if (!configuration->project.empty ()) {
    string command = "bibledit.wdb --finalize ";
    command.append (configuration->project);
    command.append (" &");
    system (command.c_str());
  }
  // Stop possible thread that is displaying notes.
  stop_displaying_more_notes ();
  // Vacuum, that is, optimize, the notes database.
  // It is done on shutdown of Bibledit, in the background.
  string command = "echo \"vacuum;\" | sqlite3 '" + notes_database_filename () + "' &";
  system (command.c_str ());
  // Finally destroy the configuration.
  if (configuration)
    delete configuration;
  // Destroying the window is done by gtk itself.
}


int
MainWindow::run ()
{
  return gtk_dialog_run (GTK_DIALOG (mainwindow));
}


/*
|
|
|
|
|
Initialization
|
|
|
|
|
 */


void
MainWindow::initialize_project ()
{

  // If we've no project, disables the menus and knock off.
  if (configuration->project.empty ())
    {
      enable_or_disable_widgets (false);
      return;
    }
  // Get the Scripture.
  scripture = new Scripture (configuration->project);

  // Update the titlebar.
  set_titlebar (configuration->project);

  // Enable the menus.
  enable_or_disable_widgets (true);

  // Variable needed otherwise the navigation rumbles too much.
  driving_book = true;

  // Put the names of the books in the combo.
  // Put them also in a vector, because the books cannot be extracted from the widget itself.
  GList *combo_book_items = NULL;
  combo_book_popdown_strings.clear ();
  string gotobook;
  for (unsigned int i = 0; i < scripture->books.size (); i++) {
    combo_book_items = g_list_append (combo_book_items, (gpointer) scripture->books[i].c_str ());
    if (i == 0)
      gotobook = scripture->books[0];
    combo_book_popdown_strings.push_back (scripture->books[i]);
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_book), combo_book_items);
  g_list_free (combo_book_items);

  int bookindex = scripture->get_index_of_book (configuration->book_opened);
  if (bookindex >= 0) {
    // Set the book to the one that was opened last.
    gotobook = scripture->books[bookindex];
  }
  else {
    // If the book does not exist, reset chapter and verse.
    configuration->chapter_opened = "1";
    configuration->verse_opened = "1";
  }

  // Combo has been filled: clear the anti-rumble variable.
  driving_book = false;

  // Clear some variables so that the new book will be loaded.
  editor_current_book.clear ();
  editor_current_chapter.clear ();
  editor_current_verse.clear ();
  navigator_current_book.clear ();
  navigator_current_chapter.clear ();
  navigator_current_verse.clear ();

  // Go to the new reference.
  if (!gotobook.empty ())
    go_to_reference (gotobook, configuration->chapter_opened, configuration->verse_opened, true, false);

  // Clear the references tracker.
  track.clear ();

  // Initialize word database.
  {
    string command = "bibledit.wdb --initialize '";
    command.append (configuration->project);
    command.append ("' &");
    system (command.c_str ());
  }
  
  // Appearance of text in editor.
  // The font is now tied to the project, but also stored in the general
  // configuration.
  Project project (configuration);
  configuration->editor_font_name = project.font;
  editor_set_font ();
}


void
MainWindow::enable_or_disable_widgets (bool enable)
{
  // Depending on whether a project has been opened, make some widgets (in)sensitive.
  gtk_widget_set_sensitive (close1, enable);
  gtk_widget_set_sensitive (properties1, enable);
  gtk_widget_set_sensitive (import1, enable);
  gtk_widget_set_sensitive (menuitem_edit, enable);
  gtk_widget_set_sensitive (menuitem_view, enable);
  gtk_widget_set_sensitive (menuitem_goto, enable);
  gtk_widget_set_sensitive (compare_with1, enable);
  gtk_widget_set_sensitive (copy_project_to, enable);
  gtk_widget_set_sensitive (combo_book, enable);
  gtk_widget_set_sensitive (combo_chapter, enable);
  gtk_widget_set_sensitive (combo_verse, enable);
  if (!enable)
    {
      gtk_entry_set_text (GTK_ENTRY (combo_entry_book), "");
      gtk_entry_set_text (GTK_ENTRY (combo_entry_chapter), "");
      gtk_entry_set_text (GTK_ENTRY (combo_entry_verse), "");
    }
  gtk_widget_set_sensitive (textview_edit, enable);
  // If project is closed no references can be printed because no text is available.
  gtk_widget_set_sensitive (print_references, enable);
  // If project closed, no text can be printed.
  gtk_widget_set_sensitive (print_project, enable);
  
}


/*
|
|
|
|
|
Menu callbacks
|
|
|
|
|
 */


void MainWindow::on_open1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->open ();
}


void MainWindow::open ()
{
  OpenDialog opendialog (configuration->project);
  if (opendialog.run () == GTK_RESPONSE_OK)
    {
      close ();
      configuration->project = opendialog.project;
      initialize_project ();
    }
}


void MainWindow::on_close1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->close ();
}


void
MainWindow::close ()
{
  editor_save ();
  if (book) {
    delete book;
    book = NULL;
  }
  enable_or_disable_widgets (false);
  gtk_text_buffer_set_text (textbuffer_edit, "", -1);
  gtk_text_buffer_set_modified (textbuffer_edit, false);
  editor_current_book.clear ();
  editor_current_chapter.clear ();
  editor_current_verse.clear ();
  navigator_current_book.clear ();
  navigator_current_chapter.clear ();
  navigator_current_verse.clear ();
  gtk_statusbar_pop (GTK_STATUSBAR (statusbar1), statusbar_context_id);
  configuration->project.clear ();
  if (scripture) {
    delete scripture;
    scripture = NULL;
  }
  if (navigator_book) {
    delete navigator_book;
    navigator_book = NULL;
  }
  set_titlebar ("");
}


void MainWindow::on_new1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->newproject ();
}


void
MainWindow::newproject ()
{
  projectutils_new (configuration);
  ProjectDialog projectdialog (NEW_PROJECT);
  if (projectdialog.run () == GTK_RESPONSE_OK) {
    close ();
    configuration->project = projectdialog.newprojectname;
    initialize_project ();
  }
}


void MainWindow::on_print_project_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_print_project ();
}


void MainWindow::on_print_project ()
{
  // Save any changes.
  editor_save();
  // Run the dialog for printing the references.
  PrintProjectDialog dialog (scripture, configuration->project);
  dialog.run ();
}


void MainWindow::on_properties1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->editproject ();
}


void
MainWindow::editproject ()
{
  ProjectDialog projectdialog (configuration->project);
  if (projectdialog.run () == GTK_RESPONSE_OK)
    {
      close ();
      configuration->project = projectdialog.newprojectname;
      initialize_project ();
    }
}


void MainWindow::on_delete1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->deleteproject ();
}


void
MainWindow::deleteproject ()
{
  ProjectsDialog dialog (configuration->project);
  dialog.run ();
}


void MainWindow::on_quit1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gtk_main_quit ();
}


void MainWindow::on_view_system_log1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->viewlog ();
}


void
MainWindow::viewlog ()
{
  ShowScriptDialog showscript (LOG_FILE);
  showscript.run ();
}


void MainWindow::on_help1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  htmlbrowser (g_build_filename (PACKAGE_DATA_DIR, "index.html", NULL));
}


void MainWindow::on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->showabout ();
}


void
MainWindow::showabout ()
{
  string message = "bibledit ";
  message.append (PACKAGE_VERSION);
  message.append ("\n\n");
  message.append (PACKAGE_BUGREPORT);
  message.append ("\n");
  GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                              GTK_DIALOG_DESTROY_WITH_PARENT,
                                              GTK_MESSAGE_INFO,
                                              GTK_BUTTONS_OK,
                                              message.c_str ());
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


void MainWindow::on_undo1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_undo ();
}


void
MainWindow::menu_undo ()
{
  undo.undo (textbuffer_edit);
}


void MainWindow::on_redo1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_redo ();
}


void
MainWindow::menu_redo ()
{
  undo.redo (textbuffer_edit);
}


void MainWindow::on_edit1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_edit ();
}


void
MainWindow::menu_edit ()
{
  // Set the sensitivity of the items under the edit menu.
  gtk_widget_set_sensitive (undo1, undo.can_undo ());
  gtk_widget_set_sensitive (redo1, undo.can_redo ());
  // At present there does not seem to be a function that indicates
  // whether and which clipboard operations are possible. Therefore
  // the Copy, Paste and Cut menus are always active.

  // Enable/disable based on whether we're editing a note.
  bool enable;
  enable = (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook1)) == 1);
  // References can only be taken from a note when it is opened.
  gtk_widget_set_sensitive (get_references_from_note, enable);
}


void MainWindow::on_find1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_find ();
}


void
MainWindow::menu_find ()
{
  // Before finding, save the current file.
  editor_save ();
  // Start find dialog.
  bool is_indexing = false;
  {
    FindDialog finddialog (configuration);
    if (finddialog.run () == GTK_RESPONSE_OK)
      {
        is_indexing = finddialog.is_indexing;
        references_searchwords.clear ();
        references_searchwords.push_back (configuration->find_searchword);
        references_casesensitives.clear ();
        references_casesensitives.push_back (configuration->case_sensitive_search);
        sort_references (finddialog.results);
        References references (liststore, treeview1, treecolumn);
        references.set_references (finddialog.results, references_searchwords, references_casesensitives);
        references.fill_store ();
      }
  }
  /*
   * Give a message if the text is still being indexed.
   * If being indexed, the message also comes when there are none results,
   * because it might have been because of the inaccuracy that there are no
   * results, whereas when things were accurate, there could have been results.
   */
  if (is_indexing)
    {
      GtkWidget *dialog;
      dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                       GTK_DIALOG_DESTROY_WITH_PARENT,
                                       GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                                       "The text is still being indexed.\nTherefore the results may or may not be up to date.");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    }
}


void
MainWindow::on_find_and_replace1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_replace ();
}


void
MainWindow::menu_replace ()
{
  // Before finding, save the current file.
  editor_save ();
  // Start find/replace dialog.
  vector < string > results;
  bool is_indexing = false;
  {
    ReplaceDialog replacedialog (configuration);
    if (replacedialog.run () == GTK_RESPONSE_OK)
      {
        is_indexing = replacedialog.is_indexing;
        references_searchwords.clear ();
        references_searchwords.push_back (configuration->find_searchword);
        references_searchwords.push_back (configuration->find_replaceword);
        references_casesensitives.clear ();
        references_casesensitives.push_back (configuration->case_sensitive_search);
        references_casesensitives.push_back (configuration->case_sensitive_search);
        sort_references (replacedialog.results);
        results.assign (replacedialog.results.begin (), replacedialog.results.end ());
        References references (liststore, treeview1, treecolumn);
        references.set_references (replacedialog.results, references_searchwords, references_casesensitives);
        references.fill_store ();
      }
    else
      {
        return;
      }
  }
  /*
   * Give a message if the text is still being indexed.
   * If being indexed, the message also comes when there are none results,
   * because it might have been because of the inaccuracy that there are no
   * results, whereas when things were accurate, there could have been results.
   */
  if (is_indexing)
    {
      GtkWidget *dialog;
      dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                       GTK_DIALOG_DESTROY_WITH_PARENT,
                                       GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                                       "The text is still being indexed.\nTherefore the results may or may not be up to date.");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    }
  // Replace text.
  if (results.size ())
    {
      string prj = configuration->project;
      ReplacingDialog replacedialog (prj, results, configuration->find_searchword,
                                     configuration->find_replaceword,
                                     configuration->case_sensitive_search);
      replacedialog.run ();
      close ();
      configuration->project = prj;
      initialize_project ();
    }
  else
    {
      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_MESSAGE_INFO,
                                                  GTK_BUTTONS_OK,
                                                  "There was nothing to replace");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    }
}


void MainWindow::on_findspecial1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_findspecial ();
}


void
MainWindow::menu_findspecial ()
{
  // Before finding, save the current file.
  editor_save ();
  // Start dialog.
  bool is_indexing = false;
  {
    SearchSpecialDialog dialog (configuration, liststore);
    if (dialog.run () == GTK_RESPONSE_OK)
      {
        is_indexing = dialog.is_indexing;
        references_searchwords.push_back (configuration->find_searchword);
        references_casesensitives.push_back (configuration->case_sensitive_search);
        sort_references (dialog.results);
        References references (liststore, treeview1, treecolumn);
        references.set_references (dialog.results, references_searchwords, references_casesensitives);
        references.fill_store ();
      }
  }
  /*
   * Give a message if the text is still being indexed.
   * If being indexed, the message also comes when there are none results,
   * because it might have been because of the inaccuracy that there are no
   * results, whereas when things were accurate, there could have been results.
   */
  if (is_indexing)
    {
      GtkWidget *dialog;
      dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                       GTK_DIALOG_DESTROY_WITH_PARENT,
                                       GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                                       "The text is still being indexed.\nTherefore the results may or may not be up to date.");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    }
}


void MainWindow::on_bibledit_windows_companion1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->companion_settings ();
}


void
MainWindow::companion_settings ()
{
  CompanionDialog companiondialog (configuration);
  if (companiondialog.run () == GTK_RESPONSE_OK)
    {
      // If there is a change in the local connection, deal with it.
      if (configuration->companion_enabled != companiondialog.local)
        {
          configuration->companion_enabled = companiondialog.local;
          if (configuration->companion_enabled)
            {
              windowscompanion.Connect ("localhost", 51515);
            }
          else
            {
              windowscompanion.Disconnect ();
            }
        }
    }
}


void MainWindow::on_font1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->editor_settings ();
}


void
MainWindow::editor_settings ()
{
  GtkWidget *font_selector;
  font_selector = gtk_font_selection_dialog_new ("Editor font");
  gtk_font_selection_dialog_set_font_name (GTK_FONT_SELECTION_DIALOG (font_selector),
                                           configuration->editor_font_name.c_str ());
  gtk_widget_show (font_selector);
  int result = gtk_dialog_run (GTK_DIALOG (font_selector));
  string s = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (font_selector));
  gtk_widget_destroy (font_selector);
  if (result == GTK_RESPONSE_OK) {
    configuration->editor_font_name = s;
    editor_set_font ();
    Project project (configuration);
    project.font = configuration->editor_font_name;
  }
}


void MainWindow::on_import1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->menu_import ();
}


void
MainWindow::menu_import ()
{
  ImportSfmDialog dialog (configuration->project);
  int result = dialog.run ();
  if (result == GTK_RESPONSE_APPLY)
    {
      string prj = configuration->project;
      close ();
      configuration->project = prj;
      initialize_project ();
    }
}


gboolean MainWindow::on_mainwindow_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
/*
  This solves a bug:
  When quitting the program though the menu, all went fine.
  But when quitting the program by clicking on the cross at the top of the program,
  a segmentation fault occurred.
  This segmentation occurred in the destructor of the object MainWindow.
  In the destructor properties of GTK were accessed, which obviously were 
  already in the process of being destroyed.
  This is the solution:
  When the window receives the delete_event (somebody clicks on the cross to
  close the program), the window is not closed, but instead a timeout is 
  installed that calls qtk_main_quit after a short while.
  The destructor of the object MainWindow can not access the properties of GTK
  without a segmentation fault.
  */
  g_timeout_add (10, GSourceFunc (gtk_main_quit), user_data);
  return true;
}


void MainWindow::on_insert1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_menu_insert ();
}


void MainWindow::on_menu_insert ()
// Sets the labels of the underlying menu items right.
{
  // Write the proper labels.
  string std_txt = "Standard text ";
  ustring label;
  label = std_txt + "_1: " + configuration->edit_note_standard_text_one;
  gtk_label_set_text_with_mnemonic (GTK_LABEL (gtk_bin_get_child (GTK_BIN (standard_text_1))), label.c_str());
  label = std_txt + "_2: " + configuration->edit_note_standard_text_two;
  gtk_label_set_text_with_mnemonic (GTK_LABEL (gtk_bin_get_child (GTK_BIN (standard_text_2))), label.c_str());
  label = std_txt + "_3: " + configuration->edit_note_standard_text_three;
  gtk_label_set_text_with_mnemonic (GTK_LABEL (gtk_bin_get_child (GTK_BIN (standard_text_3))), label.c_str());
  label = std_txt + "_4: " + configuration->edit_note_standard_text_four;
  gtk_label_set_text_with_mnemonic (GTK_LABEL (gtk_bin_get_child (GTK_BIN (standard_text_4))), label.c_str());
  // Enable or disable depending on situation.
  bool enable;
  enable = (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook1)) == 1);
  gtk_widget_set_sensitive (standard_text_1, enable);
  gtk_widget_set_sensitive (standard_text_2, enable);
  gtk_widget_set_sensitive (standard_text_3, enable);
  gtk_widget_set_sensitive (standard_text_4, enable);
}


void MainWindow::on_menuitem_view_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_menuitem_view ();
}


void MainWindow::on_menuitem_view ()
{
  // Enable or disable items under the "View" menu.
  bool enable;
  enable = (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook1)) == 1);
  // More info on a note can only be given when the note is being edited.
  gtk_widget_set_sensitive (more_info_on_current_note1, enable);
}


void MainWindow::on_notes_preferences_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_notes_preferences ();
}


void MainWindow::on_notes_preferences ()
{
  NotesDialog dialog (configuration);
  dialog.run();
}


void MainWindow::on_copy_project_to_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_copy_project_to ();
}


void MainWindow::on_copy_project_to ()
{
  EntryDialog dialog ("New project name", 
                      "Enter a name of a non-existent project\nwhere this project will be copied to.",
                      "");
  if (dialog.run () == GTK_RESPONSE_OK) {
    // Get all current projects, and see whether the name the user wants to copy
    // this project to already exists.
    Projects projects (directories_get_projects());
    set<ustring> allprojects (projects.projects.begin(), projects.projects.end());
    if (allprojects.find (dialog.entered_value) == allprojects.end()) {
      // Ok, go ahead with the copy.
      projectutils_copy_project (configuration->project, dialog.entered_value);
      // Give message when through.
      string message;
      message.append ("The project has been copied to a new project\n");
      message.append ("named ");
      message.append (dialog.entered_value);
      message.append (".");
      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, message.c_str ());
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    } else {
      // No, give message that project exists.
      string error = "Project ";
      error.append (dialog.entered_value);
      error.append (" already exists.");
      error.append ("\nIf you still intend to copy the project,");
      error.append ("\ndelete project ");
      error.append (dialog.entered_value);
      error.append (" first.");
      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error.c_str ());
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
    }
  }
}


void MainWindow::on_compare_with1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_compare_with ();
}


void MainWindow::on_compare_with ()
{
  editor_save();
  CompareDialog dialog (configuration->project);
  dialog.run ();
}


/*
|
|
|
|
|
Navigation
|
|
|
|
|
 */


void MainWindow::on_combo_entry_book_changed (GtkEditable * editable, gpointer user_data)
{
  ((MainWindow *) user_data)->on_combo_book_changed ();
}


void
MainWindow::on_combo_book_changed ()
{
  if (driving_book)
    return;
  string book = gtk_entry_get_text (GTK_ENTRY (combo_entry_book));
  if (book.empty ())
    return;
  go_to_reference (book, "", "", false, false);
}


void MainWindow::on_combo_entry_chapter_changed (GtkEditable * editable, gpointer user_data)
{
  ((MainWindow *) user_data)->on_combo_chapter_changed ();
}


void
MainWindow::on_combo_chapter_changed ()
{
  if (driving_chapter)
    return;
  string chapter = gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter));
  if (chapter.empty ())
    return;
  go_to_reference ("", chapter, "", false, false);
}


void MainWindow::on_combo_entry_verse_changed (GtkEditable * editable, gpointer user_data)
{
  ((MainWindow *) user_data)->on_combo_verse_changed ();
}


void
MainWindow::on_combo_verse_changed ()
{
  if (driving_verse)
    return;
  string verse = gtk_entry_get_text (GTK_ENTRY (combo_entry_verse));
  if (verse.empty ())
    return;
  go_to_reference ("", "", verse, false, false);
}


void MainWindow::on_next_verse1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_next_verse ();
}


void
MainWindow::goto_next_verse ()
{
  set_next_or_previous_item (GTK_COMBO (combo_verse), gtk_entry_get_text (GTK_ENTRY (combo_entry_verse)), true);
}


void MainWindow::on_previous_verse1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_previous_verse ();
}


void
MainWindow::goto_previous_verse ()
{
  set_next_or_previous_item (GTK_COMBO (combo_verse), gtk_entry_get_text (GTK_ENTRY (combo_entry_verse)), false);
}


void MainWindow::on_next_chapter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_next_chapter ();
}


void
MainWindow::goto_next_chapter ()
{
  set_next_or_previous_item (GTK_COMBO (combo_chapter), gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter)), true);
}


void MainWindow::on_previous_chapter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_previous_chapter ();
}


void
MainWindow::goto_previous_chapter ()
{
  set_next_or_previous_item (GTK_COMBO (combo_chapter), gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter)), false);
}


void MainWindow::on_next_book1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_next_book ();
}


void
MainWindow::goto_next_book ()
{
  set_next_or_previous_item (GTK_COMBO (combo_book), gtk_entry_get_text (GTK_ENTRY (combo_entry_book)), true);
}


void MainWindow::on_previous_book1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_previous_book ();
}


void
MainWindow::goto_previous_book ()
{
  set_next_or_previous_item (GTK_COMBO (combo_book), gtk_entry_get_text (GTK_ENTRY (combo_entry_book)), false);
}


void MainWindow::on_reference1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_reference ();
}


void
MainWindow::goto_reference ()
{
  GotoReferenceDialog dialog (navigator_current_book, navigator_current_chapter,
                              navigator_current_verse);
  if (dialog.run () == GTK_RESPONSE_OK)
    {
      if (dialog.newreference)
        go_to_reference (dialog.newbook, dialog.newchapter, dialog.newverse, true, false);
    }
}


void
MainWindow::go_to_reference (const string & gotobook, const string & gotochapter,
                             const string & gotoverse, bool jump, bool highlight)
{
  /*
   * This function makes the editor to go to a new Bible reference.
   * 
   * Going to a reference is not done immediately on request. It is instead done by a scheduler.
   * This is done because at times the program generates a lot of requests at once.
   * If the edit control is then loaded each time, and the cursor positioned,
   * this confuses Gtk when the requests come too soon one after the other.
   * So this function only stores the requested references, and a timer will carry
   * this request out later. 
   * 
   * If the previously displayed reference was edited and therefore changed,
   * it will save the file.
   * 
   * The "jump" variabele needs to be set when any arbitrary jump is made.
   * It is related to some internal logic.
   */


  // If we're busy bail out.
  if (go_to_reference_running)
    return;
  go_to_reference_running = true;

  try
  {
    // Decide where we're going to.
    string navigator_goto_book;
    if (gotobook.empty ())
      navigator_goto_book = navigator_current_book;
    else
      navigator_goto_book = gotobook;
    string navigator_goto_chapter;
    if (gotochapter.empty ())
      navigator_goto_chapter = navigator_current_chapter;
    else
      navigator_goto_chapter = gotochapter;
    string navigator_goto_verse;
    if (gotoverse.empty ())
      navigator_goto_verse = navigator_current_verse;
    else
      navigator_goto_verse = gotoverse;

    // Find out what needs to be changed: book, chapter and/or verse.
    bool new_book = (navigator_goto_book != navigator_current_book);
    bool new_chapter = (navigator_goto_chapter != navigator_current_chapter);
    bool new_verse = (navigator_goto_verse != navigator_current_verse);

    // Index of the book in the currently opened scripture.
    int bookindex = -1;
    if (scripture)
      bookindex = scripture->get_index_of_book (navigator_goto_book);

    // See whether the requested book exists.
    if (bookindex < 0)
      {
        // It doesn't exist, so do nothing.
        new_book = false;
        new_chapter = false;
        new_verse = false;
      }
    // Deal with a new book.
    if (new_book)
      {
        // Adjust the current indicator, and store the book for later.
        navigator_goto_book = scripture->books[bookindex];
        navigator_current_book = navigator_goto_book;
        configuration->book_opened = navigator_goto_book;
        // Set the text in the combo box.
        driving_book = true;
        gtk_entry_set_text (GTK_ENTRY (combo_entry_book), navigator_goto_book.c_str ());
        driving_book = false;
        // Load the book in memory. It'll provide data on the number of chapters and verses.
        if (navigator_book)
          delete navigator_book;
        navigator_book = new Book (scripture->paths[bookindex], false, false);
        // Put the chapters in the chapter combo box.
        // Also in a shadow vector, because the chapters cannot be obtained through the widget.
        GList *combo_chapter_items = NULL;
        combo_chapter_popdown_strings.clear ();
        for (unsigned int i = 1; i <= navigator_book->number_of_chapters; i++) {
          combo_chapter_popdown_strings.push_back (convert_to_string (i));
          combo_chapter_items = g_list_append (combo_chapter_items, (gpointer) combo_chapter_popdown_strings [i - 1].c_str ());
        }
        driving_chapter = true;
        gtk_combo_set_popdown_strings (GTK_COMBO (combo_chapter), combo_chapter_items);
        g_list_free (combo_chapter_items);
        driving_chapter = false;
        // As we've a new book, go to chapter one verse one, except when "jumping".
        if (!jump)
          {
            navigator_goto_chapter = "1";
            navigator_goto_verse = "1";
          }
        new_chapter = true;
      }
    // Bring the new chapter within the limits
    if (new_chapter)
      {
        unsigned int int_goto_chapter = convert_to_int (navigator_goto_chapter);
        if (int_goto_chapter < 1)
          navigator_goto_chapter = "1";
        if (int_goto_chapter > navigator_book->number_of_chapters)
          navigator_goto_chapter = convert_to_string (navigator_book->number_of_chapters);
      }
    // Deal with a new chapter
    if (new_chapter)
      {
        // Set the text in the combo box.
        driving_chapter = true;
        gtk_entry_set_text (GTK_ENTRY (combo_entry_chapter), navigator_goto_chapter.c_str ());
        driving_chapter = false;
        // Store the current chapter.
        navigator_current_chapter = navigator_goto_chapter;
        configuration->chapter_opened = navigator_current_chapter;
        // Put the number of verses in combo_verse
        // and in the shadow vector.
        GList *combo_verse_items = NULL;
        combo_verse_popdown_strings.clear ();
        unsigned int chapter = convert_to_int (navigator_goto_chapter);
        vector<string> versenumbers;
        if (chapter > 0) {
          unsigned int number_of_verses = navigator_book->number_of_verses (chapter);
          for (unsigned int i = 1; i <= number_of_verses; i++) {
            versenumbers.push_back (convert_to_string (i));
            combo_verse_items = g_list_append (combo_verse_items, (gpointer) versenumbers[i - 1].c_str ());
            combo_verse_popdown_strings.push_back (convert_to_string (i));
          }
          driving_verse = true;
          gtk_combo_set_popdown_strings (GTK_COMBO (combo_verse), combo_verse_items);
          g_list_free (combo_verse_items);
          driving_verse = false;
        }
        // Move to beginning of the chapter, except when "jumping"
        if (!jump)
          navigator_goto_verse = "1";
        new_verse = true;
      }
    // Bring the new verse within its limits.
    if (new_verse)
      {
        int int_goto_verse = convert_to_int (navigator_goto_verse);
        int limit = navigator_book->number_of_verses (convert_to_int (navigator_goto_chapter));
        if (int_goto_verse < 1)
          navigator_goto_verse = "1";
        if (int_goto_verse > limit)
          navigator_goto_verse = convert_to_string (limit);
      }
    // Deal with a new verse.
    if (new_verse)
      {
        // Set the text in the combo box.
        driving_verse = true;
        gtk_entry_set_text (GTK_ENTRY (combo_entry_verse), navigator_goto_verse.c_str ());
        driving_verse = false;
        // Update the current verse, and store it for later also.
        navigator_current_verse = navigator_goto_verse;
        configuration->verse_opened = navigator_current_verse;
      }
    navigator_highlight = highlight;
    // If we've changed a reference: reset the timer,
    // allowing the navigation timeout to update the editor a bit later.
    if (new_book || new_chapter || new_verse)
      navigation_change_timer = 0;

  }
  catch (exception)
  {
  }

  // Ready running.
  go_to_reference_running = false;
}


bool MainWindow::mainwindow_on_navigation_timeout (gpointer data)
{
  return ((MainWindow *) data)->on_navigation_timeout ();
}


bool MainWindow::on_navigation_timeout ()
{
  /*
   * This scheduler carries out requested changes of the reference.
   * It also does related tasks.
   */

  // If F7, F8 or F9 is continually held down and it repeats,
  // we've code here to deal with that so that the interface responds
  // properly. In cases like this the change of the reference
  // is delayed until the key is released.
  navigation_change_timer++;
  if (navigation_change_timer != 4)
    return true;

  // Find out what needs to be changed: book, chapter and/or verse.
  string goto_book = gtk_entry_get_text (GTK_ENTRY (combo_entry_book));
  bool new_book = (goto_book != editor_current_book);
  editor_current_book = goto_book;
  string goto_chapter = gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter));
  bool new_chapter = (goto_chapter != editor_current_chapter);
  editor_current_chapter = goto_chapter;
  string goto_verse = gtk_entry_get_text (GTK_ENTRY (combo_entry_verse));
  bool new_verse = (goto_verse != editor_current_verse);
  editor_current_verse = goto_verse;

  // See whether the current file should be saved.
  if (new_book || new_chapter)
    if (gtk_text_buffer_get_modified (textbuffer_edit))
      editor_save ();

  // Deal with a new book.
  if (new_book)
    {
      int bookindex = scripture->get_index_of_book (goto_book);
      // Load book, do not split the lines.
      if (book)
        delete book;
      book = new Book (scripture->paths[bookindex], false, true);
      // With a new book, also load a new chapter.
      new_chapter = true;
    }
  // Deal with a new chapter.
  if (new_chapter)
    {
      // Load chapter, if need be.
      editor_load_chapter ();
      // When loading a new chapter, we've also a new verse.
      new_verse = true;
    }

  if (new_book || new_chapter || new_verse)
    {
      // Position the cursor properly.
      // The positioning will be done whenever Gtk is idle.
      // This is because sometimes Gtk is slow in loading a new chapter.
      // So if the cursor positioning is done straight after loading,
      // it might not work.
      goto_cursor_position = goto_verse;
      g_idle_add (GSourceFunc (mainwindow_editor_position_cursor_at_verse), gpointer(this));
      // Send the reference to the windows companions.
      unsigned int index = english_id_to_index (goto_book);
      string value = "gotoreference " + index_to_paratext_id (index) + " " + goto_chapter + ":" + goto_verse;
      windowscompanion.Sendln (value);
      // Update the notes view.
      notes_redisplay ();
    }
  return true;
}


void MainWindow::on_textview_edit_move_cursor (GtkTextView * textview, GtkMovementStep step, gint count,
                              gboolean extend_selection, gpointer user_data)
{
  // This function is called _before_ the actual move takes place, but we
  // do need a function called _after_ the move. To solve that here we
  // call the function after a short delay, and assume that the move
  // has taken place within that delay.
  g_timeout_add (1, GSourceFunc (mainwindow_on_cursor_moved_delayed), user_data);
}


bool MainWindow::mainwindow_on_cursor_moved_delayed (gpointer data)
{
  ((MainWindow *) data)->on_cursor_moved ();
  return false;
}


void
MainWindow::on_cursor_moved ()
{
  /*
   * When the cursor has moved, the navigation system needs to be updated
   * so that it shows the right reference. If the user was, for example
   * on MAT 1:10, and the cursor moves, the move might have brought him or her
   * to another reference, though this is not necessarily so. Therefore, as we 
   * don't know where the user is now after the cursor moved, we need to find
   * it out. The book is known, the chapter is known, because both stay the same.
   * The only thing we don't know is the verse. 
   */

  // Somebody thought that in gtkmm 2.4 the move_cursor signal was useless,
  // so removed it. As we need it here we we trying to go back to the 
  // underlying GTK2 object, and connect the signal from there.
  // on_cursor_moved_tag = g_signal_connect(textview.gobj(), "move_cursor", on_move_cursor, 0);
  // on_cursor_moved_tag = g_signal_connect(textview.gobj(), "move_cursor", *this.on_cursor_moved(), 0);
  // But this does not seem to work.
  // I feel a bit alienated from the gtkmm library because of this,
  // and consider moving to the GTK2 library.

/*
  gtkmm 2.4 removed signal_cursor_moved, so bibledit is now polling the
  line the cursor is on, and takes action when it has changed.
  */

/*
 * The above is partly true now, because we've moved to gtk+ and threw out gtkmm.
 * The polling has gone too, and the function is now called only when the cursor moved.
 */

  static int previous_cursor_line = 0;
  /* Initializing the value to 0 instead of -1 solves a Segmentation fault 
   * exception when no document was loaded.
   */
  GtkTextIter iterator;
  gtk_text_buffer_get_iter_at_mark (textbuffer_edit, &iterator,
                                    gtk_text_buffer_get_insert (textbuffer_edit));
  int line_of_cursor = gtk_text_iter_get_line (&iterator);
  if (line_of_cursor != previous_cursor_line)
    {
      previous_cursor_line = line_of_cursor;
      highlight_verse_at_cursor ();
      go_to_reference ("", "", cursor_get_verse_number (), false, false);
    }
}


void MainWindow::on_textview_edit_grab_focus (GtkWidget * widget, gpointer user_data)
{
  ((MainWindow *) user_data)->on_grab_focus ();
}


void
MainWindow::on_grab_focus ()
{
  // The purpose of this is to adjust the reference to the new position,
  // wherever the user clicks in the text.
  if (focus_programmatically_being_grabbed)
    return;
  g_idle_add (GSourceFunc (mainwindow_on_grab_focus_delayed_handler), gpointer(this));
}


string MainWindow::cursor_get_verse_number ()
{
  /*
   * This gets the verse number the cursor is on.
   * We will find the verse out by doing the following things:
   * 1. Get the line the cursor is on. If it has a verse marker, we then can get
   * the verse number from there.
   * 2. If we did not find a verse number, get the previous line, and see what
   * information is there.
   * 3. Keep doing step 2 until we either have the information we want, or we have
   * arrived at the beginning of the file.
   * 4. If we are at the start of the file, assume verse one.
   */
  // Get the line the cursor is on.
  GtkTextIter
    iterator;
  gtk_text_buffer_get_iter_at_mark (textbuffer_edit, &iterator,
                                    gtk_text_buffer_get_insert (textbuffer_edit));
  int
    line_of_cursor = gtk_text_iter_get_line (&iterator);
  // If no verse information is going to be found, assume verse one.
  string
    verse = "1";
  // Go through all lines, from the cursor downwards, and check whether verse information is there.
  GtkTextIter
    endofline;
  Usfm
  usfm (0);
  for (int i = line_of_cursor; i >= 0; i--)
    {
      gtk_text_buffer_get_iter_at_line (textbuffer_edit, &iterator, i);
      gtk_text_buffer_get_iter_at_line (textbuffer_edit, &endofline, i + 1);
      gtk_text_iter_backward_char (&endofline);
      ustring
        s = gtk_text_buffer_get_text (textbuffer_edit, &iterator, &endofline, false);
      ustring
        marker = usfm_extract_marker (s);
      if (usfm.is_verse (marker))
        {
          verse = number_in_string (s);
          break;
        }
    }
  return verse;
}


bool MainWindow::mainwindow_editor_position_cursor_at_verse (gpointer data)
{
  return ((MainWindow *) data)->editor_position_cursor_at_verse ();
}


bool MainWindow::editor_position_cursor_at_verse ()
{
  /*
   * This function moves the cursor of the editor to the verse stored before.
   */

  // We will not move the cursor or scroll to it when the cursor is already on the right verse.
  bool
    b = (goto_cursor_position.compare (cursor_get_verse_number ()));
  // When the cursor is at the end of the buffer, it usually means that the buffer has just been filled.
  // So the cursor does need to be repositioned in this case.
  GtkTextIter
    cursor_iterator;
  gtk_text_buffer_get_iter_at_mark (textbuffer_edit, &cursor_iterator,
                                    gtk_text_buffer_get_insert (textbuffer_edit));
  GtkTextIter
    enditer;
  gtk_text_buffer_get_end_iter (textbuffer_edit, &enditer);
  if (gtk_text_iter_compare (&cursor_iterator, &enditer) == 0)
    b = true;
  if (b)
    {
      // Grab focus here to get the scrolling done properly, and the user can type in it.
      focus_programmatically_being_grabbed = true;
      gtk_widget_grab_focus (textview_edit);
      focus_programmatically_being_grabbed = false;
      // Go through all the lines and find out about the verse.
      Usfm
      usfm (0);
      for (int i = 0; i < gtk_text_buffer_get_line_count (textbuffer_edit); i++)
        {
          GtkTextIter
            begin;
          gtk_text_buffer_get_iter_at_line (textbuffer_edit, &begin, i);
          GtkTextIter
            end;
          gtk_text_buffer_get_iter_at_line (textbuffer_edit, &end, i + 1);
          ustring
            line = gtk_text_buffer_get_text (textbuffer_edit, &begin, &end, false);
          ustring
            marker = usfm_extract_marker (line);
          if (usfm.is_verse (marker))
            {
              ustring
                verse = number_in_string (line);
              if (verse == goto_cursor_position)
                {
                  // Move the cursor to it.
                  // Move it to the beginning of the text, if there is any.
                  int
                    offset = 3 + marker.length () + verse.length ();
                  ustring
                    s = gtk_text_buffer_get_text (textbuffer_edit, &begin, &end, false);
                  int
                    linelength = s.length ();
                  linelength--;
                  if (offset >= linelength)
                    offset = linelength;
                  gtk_text_buffer_get_iter_at_line_offset (textbuffer_edit, &begin, i, offset);
                  gtk_text_buffer_place_cursor (textbuffer_edit, &begin);
                  // Scroll also to it. It will scroll to the beginning of the text after the verse marker.
                  // Alignment is needed to put the line being edited near the top of the window.
                  gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (textview_edit), &begin, 0.1, false,
                                                0, 0);
                  // Highlight the verse we moved to (remove any previous tags)
                  highlight_verse_at_cursor ();
                  break;
                }
            }
        }
      // Highlight searchwords if need be.
      highlight_searchwords (convert_to_int (goto_cursor_position));
    }
  return false;
}


/*
|
|
|
|
|
Tracker
|
|
|
|
|
 */


void MainWindow::on_button_forward_clicked (GtkButton * button, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_next_reference ();
}


void
MainWindow::goto_next_reference ()
{
  set_next_or_previous_reference (true);
}


void MainWindow::on_button_back_clicked (GtkButton * button, gpointer user_data)
{
  ((MainWindow *) user_data)->goto_previous_reference ();
}


void
MainWindow::goto_previous_reference ()
{
  set_next_or_previous_reference (false);
}


void
MainWindow::set_next_or_previous_item (GtkCombo * combo, const string & current_text, bool next)
{
  // This function takes the next (or previous) value in the dropdown list of the combo box
  // and positions the editor there.
  vector < string > strings;
  if (GTK_WIDGET (combo) == combo_book)
    for (unsigned int i = 0; i < combo_book_popdown_strings.size (); i++)
      strings.push_back (combo_book_popdown_strings[i]);
  if (GTK_WIDGET (combo) == combo_chapter)
    for (unsigned int i = 0; i < combo_chapter_popdown_strings.size (); i++)
      strings.push_back (combo_chapter_popdown_strings[i]);
  if (GTK_WIDGET (combo) == combo_verse)
    for (unsigned int i = 0; i < combo_verse_popdown_strings.size (); i++)
      strings.push_back (combo_verse_popdown_strings[i]);
  if (strings.size () == 0)
    return;
  unsigned int index = 0;
  for (unsigned int i = 0; i < strings.size (); i++)
    {
      if (current_text == strings[i])
        index = i;
    }
  if (next)
    {
      if (index == (strings.size () - 1))
        return;
      index++;
    }
  else
    {
      if (index == 0)
        return;
      index--;
    }
  string combo_text;
  combo_text = strings[index];
  // Go to the new reference.
  if (GTK_WIDGET (combo) == combo_book)
    go_to_reference (combo_text, "", "", false, false);
  if (GTK_WIDGET (combo) == combo_chapter)
    go_to_reference ("", combo_text, "", false, false);
  if (GTK_WIDGET (combo) == combo_verse)
    go_to_reference ("", "", combo_text, false, false);
}


void
MainWindow::set_sensitivity_of_navigator_buttons ()
{
  // Enables or disables each navigator button, depending on whether references for it are available.
  gtk_widget_set_sensitive (button_back, track.previous_reference_available ());
  gtk_widget_set_sensitive (button_forward, track.next_reference_available ());
}


bool MainWindow::mainwindow_on_tracker_timeout (gpointer data)
{
  return ((MainWindow *) data)->on_tracker_timeout ();
}


bool MainWindow::on_tracker_timeout ()
{
  // Drive the Bible reference tracker.
  track.tick (gtk_entry_get_text (GTK_ENTRY (combo_entry_book)),
              gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter)),
              gtk_entry_get_text (GTK_ENTRY (combo_entry_verse)));
  set_sensitivity_of_navigator_buttons ();
  return true;
}


void
MainWindow::set_next_or_previous_reference (bool next)
{
  string book;
  string chapter;
  string verse;
  if (next)
    track.get_next_reference (book, chapter, verse);
  else
    track.get_previous_reference (book, chapter, verse);
  go_to_reference (book, chapter, verse, true, false);
  set_sensitivity_of_navigator_buttons ();
}


bool MainWindow::mainwindow_on_grab_focus_delayed_handler (gpointer data)
{
  return ((MainWindow *) data)->on_grab_focus_delayed_handler ();
}


bool MainWindow::on_grab_focus_delayed_handler ()
{
  /*
   * If the user clicks in the editor window, and straight after that
   * the verse where the cursor is, is asked, we get a reply 
   * indicating the verse where the cursor was prior to clicking.
   * This delayed handler solves that.
   */
  go_to_reference ("", "", cursor_get_verse_number (), false, false);
  highlight_verse_at_cursor ();
  return false;
}


void
MainWindow::highlight_verse_at_cursor ()
{
  // Remove previous highlight, if any.
  GtkTextIter startiter;
  GtkTextIter enditer;
  gtk_text_buffer_get_start_iter (textbuffer_edit, &startiter);
  gtk_text_buffer_get_end_iter (textbuffer_edit, &enditer);
  gtk_text_buffer_remove_tag (textbuffer_edit, editor_line_tag, &startiter, &enditer);
  // Hightlight the line number the cursor is on.
  gtk_text_buffer_get_iter_at_mark (textbuffer_edit, &startiter,
                                    gtk_text_buffer_get_insert (textbuffer_edit));
  int line_of_cursor = gtk_text_iter_get_line (&startiter);
  gtk_text_buffer_get_iter_at_line (textbuffer_edit, &startiter, line_of_cursor);
  gtk_text_buffer_get_iter_at_line (textbuffer_edit, &enditer, ++line_of_cursor);
  gtk_text_buffer_apply_tag (textbuffer_edit, editor_line_tag, &startiter, &enditer);
}


/*
|
|
|
|
|
Editing
|
|
|
|
|
 */


void
MainWindow::editor_load_chapter ()
{
  /*
   * Loads the chapter in the editor.
   */
  // Set the new text.
  vector < ustring > lines;
  book->get_chapter (convert_to_int (gtk_entry_get_text (GTK_ENTRY (combo_entry_chapter))), lines);
  ustring text;
  for (unsigned int i = 0; i < lines.size (); i++)
    {
      text.append (lines[i] + "\n");
    }
  gtk_text_buffer_set_text (textbuffer_edit, text.c_str (), -1);
  // Set the editor ummodified.
  gtk_text_buffer_set_modified (textbuffer_edit, false);
  // Clear the undo / redo stack.
  undo.clear ();
}


void
MainWindow::editor_save ()
{
  /*
   * This will save the book if it has been changed in the editor.
   */
  // See whether it was changed.
  if (gtk_text_buffer_get_modified (textbuffer_edit))
    {
      // Collect all lines of the editor
      vector < ustring > lines;
      int number_of_lines = gtk_text_buffer_get_line_count (textbuffer_edit);
      GtkTextIter iterator;
      GtkTextIter endofline;
      for (int i = 0; i < number_of_lines; i++)
        {
          gtk_text_buffer_get_iter_at_line (textbuffer_edit, &iterator, i);
          gtk_text_buffer_get_iter_at_line (textbuffer_edit, &endofline, i + 1);
          gtk_text_iter_backward_char (&endofline);
          string s = gtk_text_buffer_get_text (textbuffer_edit,
                                               &iterator,
                                               &endofline,
                                               false);
          s = trim (s);
          if (!s.empty ())
            lines.push_back (s);
        }
      // Rejoin the lines in the chapter and save the whole book.
      book->set_chapter (lines);
      book->save ();
      // Set the buffer to unmodified.
      gtk_text_buffer_set_modified (textbuffer_edit, false);
      // Start the process that will update the word database.
      string command = "bibledit.wdb --update ";
      command.append (configuration->project);
      command.append (" '");
      command.append (index_to_biblebook (book->get_id_index ()));
      command.append ("' ");
      command.append (convert_to_string ((int) book->get_previously_presented_chapter ()));
      Shell shell (command);
      shell.set_asynchronous ();
      shell.run ();
    }
}


bool MainWindow::mainwindow_on_save_timeout (gpointer data)
{
  return ((MainWindow *) data)->on_save_timeout ();
}


bool MainWindow::on_save_timeout ()
{
  editor_save ();
  return true;
}


void
MainWindow::editor_set_font ()
{
  PangoFontDescription *font_desc = NULL;
  font_desc = pango_font_description_from_string (configuration->editor_font_name.c_str ());
  // Text editor gets this font.
  gtk_widget_modify_font (textview_edit, font_desc);
  // Notes editors get this font too.
  gtk_widget_modify_font (textview_notes, font_desc);
  gtk_widget_modify_font (textview_note, font_desc);
  // Free memory.
  pango_font_description_free (font_desc);
}


/*
|
|
|
|
|
Clipboard
|
|
|
|
|
 */


void MainWindow::on_cut1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_cut ();
}


void
MainWindow::on_cut ()
{
  GtkClipboard *clipboard;
  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (GTK_WIDGET_HAS_FOCUS (textview_edit))
    gtk_text_buffer_cut_clipboard (textbuffer_edit, clipboard, true);
  if (GTK_WIDGET_HAS_FOCUS (textview_notes))
    gtk_text_buffer_cut_clipboard (textbuffer_notes, clipboard, true);
  if (GTK_WIDGET_HAS_FOCUS (textview_note))
    gtk_text_buffer_cut_clipboard (textbuffer_note, clipboard, true);
}


void MainWindow::on_copy1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_copy ();
}


void
MainWindow::on_copy ()
{
  GtkClipboard *clipboard;
  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (GTK_WIDGET_HAS_FOCUS (textview_edit))
    gtk_text_buffer_copy_clipboard (textbuffer_edit, clipboard);
  if (GTK_WIDGET_HAS_FOCUS (textview_notes))
    gtk_text_buffer_copy_clipboard (textbuffer_notes, clipboard);
  if (GTK_WIDGET_HAS_FOCUS (textview_note))
    gtk_text_buffer_copy_clipboard (textbuffer_note, clipboard);
}


void MainWindow::on_paste1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_paste ();
}


void
MainWindow::on_paste ()
{
  GtkClipboard *clipboard;
  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (GTK_WIDGET_HAS_FOCUS (textview_edit))
    gtk_text_buffer_paste_clipboard (textbuffer_edit, clipboard, NULL, true);
  if (GTK_WIDGET_HAS_FOCUS (textview_notes))
    gtk_text_buffer_paste_clipboard (textbuffer_notes, clipboard, NULL, true);
  if (GTK_WIDGET_HAS_FOCUS (textview_note))
    gtk_text_buffer_paste_clipboard (textbuffer_note, clipboard, NULL, true);
}


/*
|
|
|
|
|
Backups
|
|
|
|
|
 */


/*
|
|
|
|
|
List store and reference handling
|
|
|
|
|
 */


gboolean MainWindow::on_treeview1_key_press_event (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
{
  /* Pressing Return on the keyboard, or Enter on the numerical keypad
   * make us go to the reference.
   */
  if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
    ((MainWindow *) user_data)->on_list_goto ();
  // Pressing Delete takes the reference(s) out that have been selected.
  if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
    ((MainWindow *) user_data)->on_delete_references ();
  return FALSE;
}


gboolean MainWindow::on_treeview1_button_press_event (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
  // Double-clicking a references makes us go to the reference.
  if (event->type == GDK_2BUTTON_PRESS)
    ((MainWindow *) user_data)->on_list_goto ();
  return FALSE;
}


void
MainWindow::on_list_goto ()
{
  /*
   * Handler for when the user pressed Enter in the list and wants to go to a reference.
   */
  try
  {
    // Get the string the user selected.
    GtkTreeSelection *selection;
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
    gtk_tree_selection_selected_foreach (selection, mainwindow_selection_foreach_function, gpointer(this));
    // Decode into book, chapter and verse.
    string book;
    string chapter;
    string verse;
    decode_reference (selected_reference, book, chapter, verse);
    // Jump to the reference.
    go_to_reference (book, chapter, verse, true, true);
  }
  catch (exception & ex)
  {
    cerr << ex.what () << endl;
  }
}


void MainWindow::mainwindow_selection_foreach_function (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
  gchar *book;
  gtk_tree_model_get (model, iter, 0, &book, -1);
  ((MainWindow *) data)->selected_reference = book;
  g_free (book);
}


void MainWindow::on_open_references1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_open_references ();
}


void
MainWindow::on_open_references ()
{
  GtkWidget *file_selector;
  file_selector = gtk_file_selection_new ("Please choose a file");
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector),
                                   configuration->references_file.c_str ());
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  string s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  ustring searchword;
  if (result == GTK_RESPONSE_OK) {
    EntryDialog dialog ("Enter searchword", "Which word do these references search for?", configuration->import_references_searchword);
    result = dialog.run();
    if (result == GTK_RESPONSE_OK) {
      configuration->import_references_searchword = dialog.entered_value;
      searchword = dialog.entered_value;
    }
  }
  if (result == GTK_RESPONSE_OK) {
    configuration->references_file = s;
    References references (liststore, treeview1, treecolumn);
    references.load (configuration->references_file);
    references.get_references (references_searchwords, references_casesensitives);
    references.fill_store ();
    if (!searchword.empty()) {
      references_searchwords.clear();
      references_searchwords.push_back(searchword);
      references_casesensitives.clear();
      references_casesensitives.push_back(false);
    }
  }
}


void MainWindow::on_references_save_as_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_save_references ();
}


void
MainWindow::on_save_references ()
{
  try
  {
    GtkWidget *file_selector;
    file_selector = gtk_file_selection_new ("Please choose a file");
    gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector),
                                     configuration->references_file.c_str ());
    gtk_widget_show (file_selector);
    int result = gtk_dialog_run (GTK_DIALOG (file_selector));
    string s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
    gtk_widget_destroy (file_selector);
    if (result == GTK_RESPONSE_OK)
      {
        configuration->references_file = s;
        References references (liststore, treeview1, treecolumn);
        // Hack: set references with a dummy, then load the real ones from the editor.
        vector < string > dummy;
        references.set_references (dummy, references_searchwords, references_casesensitives);
        references.get_loaded ();
        references.save (s);
      }
  }
  catch (exception & ex)
  {
    cerr << "Saving references: " << ex.what () << endl;
  }
}


void MainWindow::on_print_references_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_print_references ();
}


void MainWindow::on_print_references ()
{
  // Save any changes.
  editor_save();
  // Load refs from the editor.
  References references (liststore, treeview1, treecolumn);
  references.get_loaded();
  vector<string> refs;
  references.get_references (refs);
  // Run the dialog for printing the references.
  PrintReferencesDialog dialog (configuration->project, refs, references_searchwords, references_casesensitives);
  dialog.run ();
}


void MainWindow::on_close_references_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_clear_references ();
}


void
MainWindow::on_clear_references ()
{
  References references (liststore, treeview1, treecolumn);
  references.fill_store ();
}


void MainWindow::on_delete_references_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_delete_references ();
}


void
MainWindow::on_delete_references ()
{
  // Delete each selected row.
  GtkTreeSelection *selection;
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
  vector < GtkTreeIter > iters;
  gtk_tree_selection_selected_foreach (selection, MainWindow::on_delete_reference, gpointer (&iters));
  for (unsigned int i = 0; i < iters.size (); i++)
    {
      GtkTreeIter iter = iters[i];
      gtk_list_store_remove (liststore, &iter);
    }
  // After deleting reference(s) save any remaining references to disk.
  References references (liststore, treeview1, treecolumn);
  // There was a bug that highlighting no longer worked after deleting references.
  // Solved by having the object load all searchwords before saving them.
  vector < string > dummy;
  references.set_references (dummy, references_searchwords, references_casesensitives);
  references.get_loaded ();
  references.save ("");
  // Update the header with the number of references.
  references.set_header ();
}


void
MainWindow::on_delete_reference (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter,
                                 gpointer data)
{
  ((vector < GtkTreeIter > *)data)->push_back (*iter);
}


void MainWindow::on_next_reference1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_next_reference ();
}


void
MainWindow::on_next_reference ()
{
  /* 
   * This goes to the next reference, if there is any.
   * If no item has been selected it chooses the first, if it's there.
   */
  // Select next reference in the list.
  References references (liststore, treeview1, treecolumn);
  references.goto_next ();
  // Actually open the reference in the editor.
  on_list_goto ();
}


void MainWindow::on_previous_reference1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_previous_reference ();
}


void
MainWindow::on_previous_reference ()
{
  /* 
   * This goes to the previous reference, if there is any.
   * If no item has been selected it chooses the first, if it's there.
   */
  // Select previous reference in the list.
  References references (liststore, treeview1, treecolumn);
  references.goto_previous ();
  // Actually open the reference in the editor.
  on_list_goto ();
}


void
MainWindow::highlight_searchwords (int versenumber)
{
/*
	This highlights all the words in the verse that agree to the word
	we search for.
*/
  if (navigator_highlight)
    {
      // Remove previous highlight, if any.
      GtkTextIter startiter;
      GtkTextIter enditer;
      gtk_text_buffer_get_start_iter (textbuffer_edit, &startiter);
      gtk_text_buffer_get_end_iter (textbuffer_edit, &enditer);
      gtk_text_buffer_remove_tag (textbuffer_edit, editor_reference_tag, &startiter, &enditer);
      // The word(s) to search for.
      for (unsigned int i = 0; i < references_searchwords.size (); i++)
        {
          ustring searchword = references_searchwords[i];
          bool casesensitive = references_casesensitives[i];
          if (!casesensitive)
            searchword = searchword.casefold ();
          // Highlight words in the line the cursor is on.
          bool cursor_set = false;
          GtkTextIter iterator;
          gtk_text_buffer_get_iter_at_mark (textbuffer_edit, &iterator,
                                            gtk_text_buffer_get_insert (textbuffer_edit));
          int line_of_cursor = gtk_text_iter_get_line (&iterator);
          bool is_verse;
          highlight_searchwords_internal (line_of_cursor, versenumber, searchword, cursor_set,
                                          is_verse, casesensitive);
          // See whether we have lines before this verse that is not a verse number,
          // and highlight words there too.
          int lower_bound = line_of_cursor - 5;
          lower_bound = CLAMP (lower_bound, 0, line_of_cursor);
          int higher_bound = line_of_cursor - 1;
          higher_bound = CLAMP (higher_bound, 0, line_of_cursor);
          is_verse = false;
          for (int i = higher_bound; i >= lower_bound; i--)
            if (!is_verse)
              highlight_searchwords_internal (i, versenumber, searchword, cursor_set, is_verse,
                                              casesensitive);
          // Do the same for some lines after this verse.
          int hi_line = gtk_text_buffer_get_line_count (textbuffer_edit) - 1;
          lower_bound = line_of_cursor + 1;
          lower_bound = CLAMP (lower_bound, line_of_cursor, hi_line);
          higher_bound = line_of_cursor + 5;
          higher_bound = CLAMP (higher_bound, line_of_cursor, hi_line);
          is_verse = false;
          for (int i = lower_bound; i <= higher_bound; i++)
            if (!is_verse)
              highlight_searchwords_internal (i, versenumber, searchword, cursor_set, is_verse,
                                              casesensitive);
        }
    }
}


void
MainWindow::highlight_searchwords_internal (unsigned int linenumber, unsigned int versenumber,
                                            const string & searchword, bool & cursor_set,
                                            bool & is_verse, bool casesensitive)
{
  // Extract the line.
  GtkTextIter begin;
  gtk_text_buffer_get_iter_at_line (textbuffer_edit, &begin, linenumber);
  GtkTextIter end;
  gtk_text_buffer_get_iter_at_line (textbuffer_edit, &end, linenumber + 1);
  ustring line = gtk_text_buffer_get_text (textbuffer_edit, &begin, &end, false);
  if (!casesensitive)
    line = line.casefold ();
  // Get the verse number.
  string this_verse;
  Usfm usfm (0);
  ustring marker (line);
  marker = usfm_extract_marker (marker);
  is_verse = usfm.is_verse (marker);
  if (is_verse)
    this_verse = number_in_string (line);
  // If there is any verse number and it is different from the one we have, return.
  if (is_verse)
    if (convert_to_int (this_verse) != versenumber)
      return;
  // Find all places in this line that have the search word.
  unsigned int offposition = line.find (searchword);
  while (offposition != string::npos)
    {
      // Highlight the word.
      gtk_text_buffer_get_iter_at_line_offset (textbuffer_edit, &begin, linenumber, offposition);
      gtk_text_buffer_get_iter_at_line_offset (textbuffer_edit, &end, linenumber,
                                               offposition + searchword.length ());
      gtk_text_buffer_apply_tag (textbuffer_edit, editor_reference_tag, &begin, &end);
      if (!cursor_set)
        {
          gtk_text_buffer_place_cursor (textbuffer_edit, &begin);
          cursor_set = true;
        }
      offposition = line.find (searchword, offposition + searchword.length ());
    }
}


/*
|
|
|
|
|
Undo / Redo
|
|
|
|
|
 */


void MainWindow::on_textbuffer_edit_insert_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, gchar * arg2,
                                gint arg3, gpointer user_data)
{
  // As the documentation of this callback is not so good (gtk+ 2.6 was checked),
  // here follows some extra info.
  // textbuffer is the buffer where the text was inserted.
  // arg1 is the iterator pointing to the location where the insert was done.
  // arg2 is the string that was inserted.
  // arg3 is the length of the string that was inserted.
  ustring text (arg2);
  ((MainWindow *) user_data)->on_insert (textbuffer, arg1, text, text.length ());
}


void MainWindow::on_textbuffer_edit_delete_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, GtkTextIter * arg2,
                                gpointer user_data)
{
  // As the documentation of this callback is not so good (gtk+ 2.6 was checked),
  // here follows some extra info.
  // textbuffer is the buffer where text was deleted from.
  // arg1 is the iterator pointing to the beginning of the deleted text.
  // arg2 is the iterator pointing to the end of the deleted text.
  ((MainWindow *) user_data)->on_erase (textbuffer, arg1, arg2);
}


void
MainWindow::on_insert (GtkTextBuffer * textbuffer, GtkTextIter * pos, const ustring & text,
                       int bytes)
{
  undo.on_insert (pos, text, bytes);
}


void
MainWindow::on_erase (GtkTextBuffer * textbuffer, GtkTextIter * range_begin,
                      GtkTextIter * range_end)
{
  undo.on_erase (range_begin, range_end);
}


/*
|
|
|
|
|
Bibledit Windows Companion
|
|
|
|
|
 */


bool MainWindow::mainwindow_on_windows_companion_timeout (gpointer data)
{
  return ((MainWindow *) data)->on_windows_companion_timeout ();
}


bool MainWindow::on_windows_companion_timeout ()
{
  if (configuration->companion_enabled)
    windowscompanion.Tick ();
  return true;
}


/*
|
|
|
|
|
Title bar and status bar
|
|
|
|
|
 */


void
MainWindow::set_titlebar (const ustring& project)
{
  ustring title ("Bible Editor");
  if (project.length() > 0) title.append (" - " + project);
  gtk_window_set_title (GTK_WINDOW (mainwindow), title.c_str());
}


/*
int context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar1), "");
gtk_statusbar_push (GTK_STATUSBAR (statusbar1), context_id, configuration->project.c_str ());
*/


/*
|
|
|
|
|
Notes editor
|
|
|
|
|
 */


gboolean MainWindow::notes_motion_notify_event (GtkWidget *text_view, GdkEventMotion *event, gpointer user_data)
{
  return ((MainWindow *) user_data)->on_notes_pointer_moved (text_view, event);
}


gboolean MainWindow::on_notes_pointer_moved (GtkWidget *text_view, GdkEventMotion *event)
// Update the cursor image if the pointer moved. 
{
  gint x, y;
  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET,
                                         gint(event->x), gint(event->y), &x, &y);
  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), x, y);
  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
  return false;
}


void MainWindow::set_cursor_if_appropriate (GtkTextView *text_view, gint x, gint y)
/* Looks at all tags covering the position (x, y) in the text view, 
 * and if one of them is a link, change the cursor to the "hands" cursor
 * typically used by web browsers.
 */
{
  GSList *tags = NULL, *tagp = NULL;
  GtkTextBuffer *buffer;
  GtkTextIter iter;
  gboolean hovering = FALSE;
  buffer = gtk_text_view_get_buffer (text_view);
  gtk_text_view_get_iter_at_location (text_view, &iter, x, y);
  tags = gtk_text_iter_get_tags (&iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next) {
    GtkTextTag *tag = (GtkTextTag *) tagp->data;
    gint id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tag), "id"));
    if (id != 0) {
      hovering = TRUE;
      break;
    }
  }
  if (hovering != hovering_over_link) {
    hovering_over_link = hovering;
    if (hovering_over_link)
      gdk_window_set_cursor (gtk_text_view_get_window (text_view, GTK_TEXT_WINDOW_TEXT), hand_cursor);
    else
      gdk_window_set_cursor (gtk_text_view_get_window (text_view, GTK_TEXT_WINDOW_TEXT), regular_cursor);
  }
  if (tags) 
    g_slist_free (tags);
}


gboolean MainWindow::notes_visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event, gpointer user_data)
{
  return ((MainWindow *) user_data)->on_notes_visibility_notified (text_view, event);
}


gboolean MainWindow::on_notes_visibility_notified (GtkWidget *text_view, GdkEventVisibility *event)
/* Also update the cursor image if the window becomes visible
 * (e.g. when a window covering it got iconified).
 */
{
  gint wx, wy, bx, by;
  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);
  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), bx, by);
  return false;
}


gboolean MainWindow::notes_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  return ((MainWindow *) user_data)->on_notes_key_press_event (widget, event);
}


gboolean MainWindow::on_notes_key_press_event (GtkWidget *widget, GdkEventKey *event)
/* Links can be activated by pressing Enter.
 */
{
  GtkTextIter iter;
  GtkTextBuffer *buffer;
  switch (event->keyval) {
    case GDK_Return: 
    case GDK_KP_Enter:
      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
      gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
      notes_edit_if_link (widget, &iter);
      break;
    case GDK_Delete: 
    case GDK_KP_Delete:
      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
      gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
      notes_delete_if_link (widget, &iter);
      break;
    default:
      break;
  }
  return false;
}


void MainWindow::notes_event_after (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  ((MainWindow *) user_data)->on_notes_event_after (widget, event);  
}


void MainWindow::on_notes_event_after (GtkWidget *text_view, GdkEvent *ev)
/* Links can also be activated by clicking.
 */
{
  GtkTextIter start, end, iter;
  GtkTextBuffer *buffer;
  GdkEventButton *event;
  gint x, y;
  if (ev->type != GDK_BUTTON_RELEASE)
    return;
  event = (GdkEventButton *)ev;
  if (event->button != 1)
    return;
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  /* we shouldn't follow a link if the user has selected something */
  gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
  if (gtk_text_iter_get_offset (&start) != gtk_text_iter_get_offset (&end))
    return;
  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET,
                                         gint(event->x), gint(event->y), &x, &y);
  gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (text_view), &iter, x, y);
  notes_edit_if_link (text_view, &iter);
}


void MainWindow::notes_edit_if_link (GtkWidget *text_view, GtkTextIter *iter)
/* Looks at all tags covering the position of iter in the text view, 
 * and if one of them is a link, follow it by showing the page identified
 * by the data attached to it.
 */
{
  GSList *tags = NULL, *tagp = NULL;
  tags = gtk_text_iter_get_tags (iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next) {
    GtkTextTag *tag = (GtkTextTag *) tagp->data;
    gint id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tag), "id"));
    if (id != 0) {
      notes_fill_edit_screen (id, false);
      break;
    }
  }
  if (tags) 
    g_slist_free (tags);
}


void MainWindow::on_new_note_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_new_note ();  
}


void MainWindow::on_new_note () 
{
  // Get the unique id for the new note.
  int id = notes_database_get_unique_id ();
  // Get the current reference.
  string reference;
  // Create the new note.
  notes_fill_edit_screen (id, true);
}


void MainWindow::notes_delete_if_link (GtkWidget *text_view, GtkTextIter *iter)
/* Looks at all tags covering the position of iter in the text view, 
 * and if one of them is a link, delete it using the data attached to it.
 */
{
  GSList *tags = NULL, *tagp = NULL;
  tags = gtk_text_iter_get_tags (iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next) {
    GtkTextTag *tag = (GtkTextTag *) tagp->data;
    gint id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tag), "id"));
    if (id != 0) {
      notes_delete_one (id);
      notes_redisplay ();
      break;
    }
  }
  if (tags) 
    g_slist_free (tags);
}


void MainWindow::on_delete_note_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_delete_note();
}


void MainWindow::on_delete_note ()
{
  GtkTextIter iter;
  gtk_text_buffer_get_iter_at_mark (textbuffer_notes, &iter, gtk_text_buffer_get_insert (textbuffer_notes));
  notes_delete_if_link (textview_notes, &iter);
}


void MainWindow::on_notes_redisplay (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->notes_redisplay ();
}


void MainWindow::notes_redisplay ()
{
  // Reset the timer, so that the display will occur shortly.
  on_display_notes_timer = 0;
}


void MainWindow::on_find_in_notes1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->find_in_notes ();
}


void MainWindow::find_in_notes ()
{
  // Start dialog.
  {
    FindNoteDialog findnotedialog (configuration);
    if (findnotedialog.run () == GTK_RESPONSE_OK) {
      // Stop possible previous thread displaying notes.
      stop_displaying_more_notes ();
      // Store ids to display.
      notes_ids_to_display.assign (findnotedialog.ids.begin(), findnotedialog.ids.end());
      // Clear buffer.
      gtk_text_buffer_set_text (textbuffer_notes, "", -1);
      // Display all those ids.
      notes_ids_to_display_pointer = 0;
      g_idle_add (GSourceFunc (on_note_display_one), gpointer(this));
    }
  }
}


void MainWindow::on_import_notes_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_import_notes ();
}


void MainWindow::on_import_notes ()
{
  ImportNotesDialog dialog (0);
  if (dialog.run () == GTK_RESPONSE_APPLY) {
    notes_redisplay ();
  }
}


void MainWindow::on_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
  ((MainWindow *) user_data)->on_notes_button_cancel ();
}


void MainWindow::on_notes_button_cancel ()
{
  // Do standard functions for both ok and cancel.
  on_notes_button_ok_cancel ();
}


void MainWindow::on_button_ok_clicked (GtkButton *button, gpointer user_data)
{
  ((MainWindow *) user_data)->on_notes_button_ok ();
}


void MainWindow::on_notes_button_ok ()
{
  try
  {
    /*
    Validate and normalize the references.
    Bad ones are removed and a message will be given.
    If no valid references remain, stop the whole transaction and give a message.
    */
    string encoded_references;
    // Get and validate all references from the textview.
    {
      // Store references.
      vector<string> references;
      // Store possible messages here for later display.
      vector<string> messages;
      // Get all references from the editor.
      notes_get_references_from_editor (textbuffer_references, references, messages);
      // Sort the references so they appear nicely in the editor.
      sort_references (references);
      // Encode all references.
      for (unsigned int i = 0; i < references.size(); i++) {
        // Encode the reference.
        string book;
        string chapter;
        string verse;
        decode_reference (references[i], book, chapter, verse);
        if (!encoded_references.empty())
          encoded_references.append(" ");
        encoded_references.append (reference_to_numerical_equivalent (book, chapter, verse));
      }
      // See whether there are messages to display.
      if (messages.size() > 0) {
        string message;
        for (unsigned int i = 0; i < messages.size(); i++) {
          message.append (messages[i]);
          message.append ("\n");
        }
        GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, message.c_str ());
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
      }
    }
    // See whether any references are left. If not give a message.
    if (encoded_references.empty()) {
      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
                                                  "No references remain. Note was not stored");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      return;
    }
    // Connect to database and start transaction
    sqlite3::connection con (notes_database_filename ().c_str ());
    con.executenonquery ("begin;");
    // Delete previous data with "id".
    con.executenonquery ("delete from %s where id = %d;", TABLE_NOTES, edit_note_id);
    // Put new data in the database.
    // ID (integer), we take variabele "myid"
    // References (text), we take variabele "encoded_references"
    // Project (text)
    string project = gtk_entry_get_text (GTK_ENTRY (combo_project_entry));
    // Status (integer) This field is not used, and could be reused.
    // Referred (integer)
    int referred = notes_referred_to_index (gtk_entry_get_text (GTK_ENTRY (combo_ref_to_entry)));
    // Note (text)
    ustring note;
    {
      vector<ustring> lines;
      textbuffer_get_lines (textbuffer_note, lines);
      for (unsigned int i = 0; i < lines.size(); i++) {
        if (!note.empty())
          note.append("\n");
        note.append(lines[i]);
      }
    }
    // Trim off extra newlines at the end, and ensure it always has one.
    note = trim(note);
    note.append ("\n");
    // Apostrophies need to be doubled before storing them.
    note = double_apostrophy (note);
    // Casefolded (text)
    ustring casefolded = note.casefold();
    // Date created. Variabele note_info_date_created
    // Date modified.
    int date_modified;
    date_modified = date_time_julian_day_get_current ();
    // Username. Use: note_info_user_created
    // Insert data in database.
    con.executenonquery ("insert into %s values (%d, '%s', '%s', %d, %d, '%s', '%s', %d, %d, '%s');", TABLE_NOTES, edit_note_id, encoded_references.c_str(), project.c_str(), 0, referred, note.c_str(), casefolded.c_str(), note_info_date_created, date_modified, note_info_user_created.c_str());
    // Commit the transaction and close connection.
    con.executenonquery ("commit;");
    con.close ();
  }
  catch (exception & ex)
  {
    cerr << "Error: " << ex.what () << endl;
  }

  // Do standard functions for both ok and cancel.
  on_notes_button_ok_cancel ();
  // New note, or note edited, so display notes again.
  notes_redisplay ();
}


void MainWindow::on_notes_button_ok_cancel ()
// Functions common to both the ok and cancel buttons.
{
  // Focus widget that was focused previously.
  gtk_window_set_focus (GTK_WINDOW (mainwindow), note_widget_focused_previously);
  gtk_widget_grab_focus (note_widget_focused_previously);
  // Show the normal notes display again.
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook1), 0);
}


void MainWindow::notes_fill_edit_screen (int id, bool newnote)
/*
When a new note will be made, or an existing one edited, this function
set up the edit screen.
*/
{
  // Indicate we're filling combo boxes.
  note_edit_filling_note_controls = true;

  // Save variables.
  edit_note_id = id;

  // Save the widget that had the focus, so that when the notes had been edited,
  // the same widget gets the focus again.
  note_widget_focused_previously = gtk_window_get_focus (GTK_WINDOW (mainwindow));
  
  // GLists for filling the combo boxes.
  GList *combo_ref_to_items = NULL;
  GList *combo_project_items = NULL;

  // Fetch the data for the note from the database. And fill comboboxes.
  // Or in case of a new note, deal appropriately with that.
  try
  {
    sqlite3::connection con (notes_database_filename ().c_str ());
    sqlite3::reader reader;
    reader = con.executereader ("select reference, project, status, referred, note, created, modified, user from %s where id = %d;", TABLE_NOTES, id);
    if (reader.read () || newnote) {
      string numerical_equivalent_of_reference;
      if (newnote) {
        // New note, so get the current reference from the editor.
        numerical_equivalent_of_reference = reference_to_numerical_equivalent (editor_current_book, editor_current_chapter, editor_current_verse);
      } else {
        // Existing note, so get the reference(s) from the database.
        numerical_equivalent_of_reference = reader.getstring(0);
      }
      // Read the reference(s) and show them.
      gtk_text_buffer_set_text (textbuffer_references, "", -1);
      Parse parse (numerical_equivalent_of_reference);
      for (unsigned int i = 0; i < parse.words.size(); i++) {
        int book, chapter, verse;
        numerical_equivalent_to_reference (parse.words[i], book, chapter, verse);
        string reference;
        reference = book_chapter_verse_to_reference (book, chapter, verse);
        gtk_text_buffer_insert_at_cursor (textbuffer_references, reference.c_str(), -1);
        gtk_text_buffer_insert_at_cursor (textbuffer_references, "\n", -1);
      }
      // Read the note.
      ustring note;
      note = reader.getstring(4);
      gtk_text_buffer_set_text (textbuffer_note, note.c_str(), -1);
      // Read the status.
      // int status = reader.getint32(2);
      /*
      Fill the "referred to" combo.
      Ensure that the items to fill in the combo are living in memory 
      until after the popdown strings are set in the widget.
      Else garbage is set.
      */
      {
        vector<string> referred_to;
        for (unsigned int i = 0; i < NOTES_NUMBER_OF_REFERRED_TO_S; i++) {
          referred_to.push_back (notes_index_to_referred_to(i));
          combo_ref_to_items = g_list_append (combo_ref_to_items, (gpointer) referred_to[i].c_str());
        }
        gtk_combo_set_popdown_strings (GTK_COMBO (combo_ref_to), combo_ref_to_items);
        g_list_free (combo_ref_to_items);
      }
      // Read the "referred to" variabele.
      {
        int refto = reader.getint32(3);
        gtk_entry_set_text (GTK_ENTRY (combo_ref_to_entry), notes_index_to_referred_to(refto).c_str());
      }
      // Read the project. Fill the combo.
      {
        ustring project;
        if (newnote) {
          project = configuration->project;
        } else {
          project = reader.getstring(1);
        }
        vector<ustring> projects;
        if (project != "None")
          projects.push_back(project);
        if (configuration->project != project)
          projects.push_back(configuration->project);
        projects.push_back("None");
        for (unsigned int i = 0; i < projects.size(); i++)
          combo_project_items = g_list_append (combo_project_items, (gpointer) projects[i].c_str());
        gtk_combo_set_popdown_strings (GTK_COMBO (combo_project), combo_project_items);
        g_list_free (combo_project_items);
        gtk_entry_set_text (GTK_ENTRY (combo_project_entry), project.c_str());
      }
      // Read the date created.
      {
        // Fetch and store variabele for later use.       
        if (newnote) {
          note_info_date_created = date_time_julian_day_get_current ();
        } else {
          note_info_date_created = reader.getint32(5);
        }
      }
      // Read the date modified.
      {
        if (newnote) {
          note_info_date_modified = date_time_julian_day_get_current ();
        } else {
          note_info_date_modified = reader.getint32(6);
        }
      }
      // Read the user that created the note.
      {
        if (newnote) {
          note_info_user_created = g_get_real_name();
        } else {
          note_info_user_created = reader.getstring(7);
        }
      }
    }
    // Close all
    reader.close ();
    con.close ();
  }
  catch (exception & ex)
  {
    cerr << "Feeding notes window: " << ex.what () << endl;
  }
  
  // Switch screen to displaying the tab for editing.
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook1), 1);
  // Focus the widget the user is most likely going to type in.
  gtk_widget_grab_focus (textview_note);
  // Indicate we're ready filling combo boxes.
  note_edit_filling_note_controls = false;
  // As we've just opened the note, clear the flag about inserting dates.
  note_current_date_inserted = false;
}


void MainWindow::on_combo_entry_ref_to_changed (GtkEditable *editable, gpointer user_data)
{
  ((MainWindow *) user_data)->on_referred_to_changed ();
}


void MainWindow::on_referred_to_changed ()
{
  // When initializing the combo boxes, do nothing.
  if (note_edit_filling_note_controls) return;
  // Get the text from the referred-to entry.
  string text;
  text = gtk_entry_get_text (GTK_ENTRY (combo_ref_to_entry));
  // We get two signals, one empty, and one with the new text. Discard empty one.
  if (!text.empty()) {
    string message;
    message = "The note has been referred to the " + text;
    note_insert_date_and_text (message);
  }
}


void MainWindow::note_insert_date_and_text (const string& text)
{
  // If buffer does not end with a new line, insert one.
  GtkTextIter enditer;
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer);
  if (!gtk_text_iter_starts_line (&enditer)) {
    gtk_text_buffer_insert (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer, "\n", -1);
  }
  // Insert message at the end of the note.
  string message;
  message.append (g_get_real_name());
  message.append (": ");
  message.append (text);
  message.append (".");
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer);
  gtk_text_buffer_insert (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer, message.c_str(), -1);
}


void MainWindow::on_standard_text_1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_insert_standard_text (menuitem);
}


void MainWindow::on_standard_text_2_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_insert_standard_text (menuitem);
}


void MainWindow::on_standard_text_3_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_insert_standard_text (menuitem);
}


void MainWindow::on_standard_text_4_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_insert_standard_text (menuitem);
}


void MainWindow::on_insert_standard_text (GtkMenuItem *menuitem)
{
  // Find out which standard text to insert.
  string standardtext;
  if (menuitem == GTK_MENU_ITEM (standard_text_1)) {
    standardtext = configuration->edit_note_standard_text_one;
  } else if (menuitem == GTK_MENU_ITEM (standard_text_2)) {
    standardtext = configuration->edit_note_standard_text_two;
  } else if (menuitem == GTK_MENU_ITEM (standard_text_3)) {
    standardtext = configuration->edit_note_standard_text_three;
  } else if (menuitem == GTK_MENU_ITEM (standard_text_4)) {
    standardtext = configuration->edit_note_standard_text_four;
  }
  // Add space.
  standardtext.append (" ");
  // If buffer does not end with a new line, insert one.
  GtkTextIter enditer;
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer);
  if (!gtk_text_iter_starts_line (&enditer)) {
    gtk_text_buffer_insert (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer, "\n", -1);
  }
  // Message.
  string message;
  // Add the standard text.
  message.append (standardtext);
  // Insert message at the end of the note.
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer);
  gtk_text_buffer_insert (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_note)), &enditer, message.c_str(), -1);
}


void MainWindow::on_more_info_on_current_note1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_more_info_on_current_note1 ();
}


void MainWindow::on_more_info_on_current_note1 ()
{
  string message;
  message = "More info about this note\n\n";
  message.append ("Created on ");
  message.append (date_time_julian_human_readable(note_info_date_created));
  message.append ("\n");
  message.append ("Created by ");
  message.append (note_info_user_created);
  message.append ("\n");
  message.append ("Edited on ");
  message.append (date_time_julian_human_readable(note_info_date_modified));
  GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (mainwindow), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, message.c_str());
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


void MainWindow::stop_displaying_more_notes ()
// Stop the process of displaying notes.
{
  notes_ids_to_display.clear();      
  g_usleep (1000);
}


bool MainWindow::on_display_notes_timeout (gpointer data)
{
  return ((MainWindow *) data)->on_display_notes ();
}


bool MainWindow::on_display_notes ()
/*
The menu sometimes gives two callbacks to display notes. If notes are displayed
twice that is not efficient use of resources, and leads to a slow reaction
in the experience of the user.
To solve this problem a timer is kept running. Each time the menu wants to 
(re)display notes, it resets that timer. Only when the timer reaches a certain
value notes will be displayed. 
The overall effect is that these two callbacks result in displaying notes once,
and it happens with a short delay.
*/
{
  // See whether it's time to (re)display notes.
  on_display_notes_timer++;
  if (on_display_notes_timer == 4) {
    // Find out which notes to display.
    NotesViewSelector selector;
    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (notes_for_one_verse1)))
      selector = CurrentVerse;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (notes_for_all_verses1)))
      selector = AllNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (notes_edited_today1)))
      selector = TodaysNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_myself)))
      selector = MyNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_subteam)))
      selector = SubteamNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_team1)))
      selector = TeamNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_panel1)))
      selector = PanelNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_church1)))
      selector = ChurchNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_consultant1)))
      selector = ConsultantNotes;
    else if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (to_bible_society1)))
      selector = BibleSocietyNotes;
    else
      selector = CurrentVerse;
    // Fill the notes screen.
    // Stop possible previous thread displaying notes.
    stop_displaying_more_notes ();
    // Get actual verse.
    string reference = editor_current_book + " " + editor_current_chapter + ":" + editor_current_verse;
    // Select notes for display.  
    notes_select (selector, notes_ids_to_display, reference);
    // Clear buffer.
    gtk_text_buffer_set_text (textbuffer_notes, "", -1);
    // Display the notes. This is done when the editor is idle, and one note
    // at a time, so that the program remains responsive.
    notes_ids_to_display_pointer = 0;
    g_idle_add (GSourceFunc (on_note_display_one), gpointer(this));
    // notes_display (textview_notes, textbuffer_notes, notes_ids_to_display);
  }
  return true;
}


bool MainWindow::on_note_display_one (gpointer data)
{
  return ((MainWindow *) data)->note_display_one ();
}


bool MainWindow::note_display_one ()
{
  if (notes_ids_to_display_pointer < notes_ids_to_display.size()) {
    notes_display_one (textview_notes, textbuffer_notes, notes_ids_to_display[notes_ids_to_display_pointer]);
  }
  notes_ids_to_display_pointer++;
  if (notes_ids_to_display_pointer < notes_ids_to_display.size())
    // Not yet ready - allow for next note to be displayed.
    return true;
  else {
    // We're through. Release memory.
    notes_ids_to_display.clear();
    return false;
  }
}


void MainWindow::on_note_editor_changed (GtkEditable * editable, gpointer user_data)
{
  ((MainWindow *) user_data)->on_note_editor_changed_2 ();
}


void MainWindow::on_note_editor_changed_2 ()
{
  // When initializing the textbuffer, do nothing.
  if (note_edit_filling_note_controls) return;
  if (!note_current_date_inserted) {
    unsigned int today = date_time_julian_day_get_current ();
    if (today != note_info_date_modified) {
      // Flag should be set before inserting, else we get a callback 
      // from the inserting we do here.
      note_current_date_inserted = true;
      // If buffer does not end with a new line, insert one.
      GtkTextIter enditer;
      gtk_text_buffer_get_end_iter (textbuffer_note, &enditer);
      if (!gtk_text_iter_starts_line (&enditer)) {
        gtk_text_buffer_insert (textbuffer_note, &enditer, "\n", -1);
      }
      // Insert the date at the end of the note.
      string message = date_time_julian_human_readable (today);
      message.append ("\n");
      gtk_text_buffer_get_end_iter (textbuffer_note, &enditer);
      gtk_text_buffer_insert (textbuffer_note, &enditer, message.c_str(), -1);
    }
  }
}


void MainWindow::on_get_references_from_note_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ((MainWindow *) user_data)->on_get_references_from_note ();
}


void MainWindow::on_get_references_from_note ()
{
  // Store references.
  vector<string> references;
  // Store possible messages here, but they will be dumped.
  vector<string> messages;
  // Get all references from the editor.
  notes_get_references_from_editor (textbuffer_references, references, messages);
  // Sort the references so they appear nicely in the editor.
  sort_references (references);
  // Set none searchwords.
  references_searchwords.clear ();
  references_casesensitives.clear ();
  // Set references.
  References references2 (liststore, treeview1, treecolumn);
  references2.set_references (references, references_searchwords, references_casesensitives);
  references2.fill_store ();
}



/* Todo There is an attempt to focus the window that had focus before a note was edited. 
Make this working. */
