/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_UTILITIES_H
#define INCLUDED_UTILITIES_H


#include "libraries.h"
#include <glib.h>
#include <gtk/gtk.h>
#include "ustring.h"


string trim (const string & s);
ustring trim (const ustring & s);
string number_in_string (const string & str);
string upperCase (const string & s);
string lowerCase (const string & s);
string remove_spaces (const string & s);
void create_directory (const string & dir);
void write_lines (const string & file, vector < string > &lines);
void write_lines (const string & file, vector < ustring > &lines);
unsigned int convert_to_int (const string & str);
string convert_to_string (int i);
string convert_to_string (unsigned int &i);
string convert_to_string (bool b);
bool convert_to_bool (const string & s);
string temporary_file (const string & filename);
bool program_is_running (const string & commandline);
bool string_in_file (const string & filename, const string & line);
string string_reverse (const string & s);
ustring double_apostrophy (const ustring & line);
unsigned int file_get_modification_time (const string & filename);
void textbuffer_get_lines (GtkTextBuffer * buffer, vector<ustring>& lines);
// Todo use this utility function everywhere.


class ReadDirectories
{
public:
  ReadDirectories (const string & path, const string & prefix,
                   const string & suffix);
  ~ReadDirectories ();
   vector < string > directories;
private:
};

class ReadFiles
{
public:
  ReadFiles (const string & path, const string & prefix,
             const string & suffix);
  ~ReadFiles ();
   vector < string > files;
private:
};

class ReadText
{
public:
  ReadText (const string & file, bool silent = false);
  ~ReadText ();
   vector < string > lines;
private:
};

class WriteText
{
public:
  WriteText (const string & file, bool append = false);
  ~WriteText ();
  ofstream out;
private:
};

class Parse
{
public:
  Parse (const ustring & line, bool remove_punctuation = true);
  ~Parse ();
  vector < ustring > words;
private:
};

class ParseLine
{
public:
  ParseLine (const ustring & text);
  ~ParseLine ();
  vector <ustring> lines;
private:
};

class Shell
{
public:
  /* 
   * As this object is not able to be called while a previous instance is still
   * alive, this should be avoided. As a rule we use this Shell object only
   * for shell processes that are started regularly from the main window, but
   * not for use in dialogs, etc.
   */
  Shell (const string & command);
  ~Shell ();
  void set_asynchronous ();
  void run ();
  int result;
  string get_output ();
  vector < string > output_lines;
private:
  string mycommand;
  bool asynchronous;
};

#endif
